/*
 * Xelfi
 */

package xelfi.awt;

import java.awt.*;

/**
 * A class that produces a 3D Panel. This panel has two borders:
 * inner and outer. Each of them can be raised, lowered or none.
 * These borders has the same width.
 *
 * @version 2.1 1 Mar 1997
 * @author  Petr Hamernik
 */
public class Panel3D extends Panel {

	/** The basic color of the 3D border. Left and top sides are 
	 *  lighter and right and bottom sides are darker then this 
	 *  color. By default it is set to defaultBorderColor.
	 * @see #setBorderColor
	 * @see #getBorderColor
	 * @see #defaultBorderColor
	 */
	protected Color marginColor = null;

	/** Default color of the border. Currently lightGray.
	 * @see #setBorderColor
	 * @see #getBorderColor
	 * @see #marginColor
	 */
	public static final Color defaultBorderColor = new Color(192, 192, 192);

	/** Current width of the 3D border around the component
	 * @see #defaultBorderWidth
	 * @see #setBorderWidth
	 * @see #getBorderWidth
	 */	
	protected int margin = 0;

	/** Default value of margin. Currently 1.
	 * @see #margin
	 * @see #setBorderWidth
	 * @see #getBorderWidth
	 */
    public static final int defaultBorderWidth = 1;

	/** Caption of this panel. For setting use setCaption.
	 * @see #setCaption
	 * @see #getCaption
	 */
	protected String caption = null;

	/** None border. This constant is use for 
	 *  setting bevelInner and bevelOuter variables.
	 * @see #bevelInner
	 * @see #bevelOuter
	 */
	public static final int BV_NONE = 0;

	/** 3D lowered style of the border. This constant is use for 
	 *  setting bevelInner and bevelOuter variables.
	 * @see #bevelInner
	 * @see #bevelOuter
	 */
	public static final int BV_LOWERED = 1;
	
	/** 3D raised style of the border. This constant is use for 
	 *  setting bevelInner and bevelOuter variables.
	 * @see #bevelInner
	 * @see #bevelOuter
	 */
	public static final int BV_RAISED = 2;
	
	/** Current bevel of the inner border.
	 * @see #bevelOuter
	 * @see #BV_NONE
	 * @see #BV_LOWERED
	 * @see #BV_RAISED
	 * @see #setBevelInner
	 * @see #getBevelInner
	 */
	protected int bevelInner;
	
	/** Default value of bevelInner variable.
	 * @see #bevelInner
	 * @see #BV_RAISED
	 */
	public static final int defaultBevelInner = BV_RAISED;

	/** Current bevel of the outer border.
	 * @see #bevelInner
	 * @see #BV_NONE
	 * @see #BV_LOWERED
	 * @see #BV_RAISED
	 * @see #setBevelOuter
	 * @see #getBevelOuter
	 */
	protected int bevelOuter;

	/** Default value of bevelOuter variable.
	 * @see #bevelOuter
	 * @see #BV_LOWERED
	 */
	public static final int defaultBevelOuter = BV_LOWERED;

	/** Width between outer and inner borders.
	 * @see #defaultBevelWidth
	 * @see #setBevelWidth
	 * @see #getBevelWidth
	 */
	protected int bevelWidth;

	/** Default width between outer and inner borders. Currently 0.
	 * @see #bevelWidth
	 * @see #setBevelWidth
	 * @see #getBevelWidth
	 */
	public static final int defaultBevelWidth = 0;
		
	/** Constructs new Panel3D with all default values. */
    public Panel3D() {
	    setDefaults();
    }

    private void setDefaults() {
        margin = defaultBorderWidth;
        marginColor = defaultBorderColor;
		bevelInner = defaultBevelInner;
		bevelOuter = defaultBevelOuter;
		bevelWidth = defaultBevelWidth;
		caption = "";
    }

	/** Returns the insets of the Panel3D. The insets indicate 
	 *  the size of the border of the Panel3D. 
	 */
	public Insets insets() {
		int a = bevelWidth;
		int b = getCaptionDim().height;
		if (bevelInner != BV_NONE)
			a += margin;
		if (bevelOuter != BV_NONE)
			a += margin;
		b -= a;
		if (b < 0)
			b = 0;
		return new Insets(a+b,a,a,a);
	}

	/** Sets new style of the inner bevel.
	 * @param x New style.
	 * @exception ArrayIndexOutOfBoundsException If the value was invalid.
	 * @see #bevelOuter
	 * @see #BV_NONE
	 * @see #BV_LOWERED
	 * @see #BV_RAISED
	 * @see #getBevelInner
	 */
	public void setBevelInner(int x) throws ArrayIndexOutOfBoundsException {
		if ((x < 0)||(x > 2))
	        throw new ArrayIndexOutOfBoundsException();
		bevelInner = x;
		repaint();
	}

	/** Gets the current style of the inner bevel.
	 * @return Current style.
	 * @see #bevelOuter
	 * @see #BV_NONE
	 * @see #BV_LOWERED
	 * @see #BV_RAISED
	 * @see #setBevelInner
	 */
	public int getBevelInner() {
		return bevelInner;
	}

	/** Sets new style of the outer bevel.
	 * @param x New style.
	 * @exception ArrayIndexOutOfBoundsException If the value was invalid.
	 * @see #bevelInner
	 * @see #BV_NONE
	 * @see #BV_LOWERED
	 * @see #BV_RAISED
	 * @see #getBevelOuter
	 */
	public void setBevelOuter(int x) throws ArrayIndexOutOfBoundsException {
		if ((x < 0)||(x > 2))
	        throw new ArrayIndexOutOfBoundsException();
		bevelOuter = x;
		repaint();
	}

	/** Gets the current style of the outer bevel.
	 * @return Current style.
	 * @see #bevelInner
	 * @see #BV_NONE
	 * @see #BV_LOWERED
	 * @see #BV_RAISED
	 * @see #setBevelOuter
	 */
	public int getBevelOuter() {
		return bevelOuter;
	}

	/** Sets the new caption.
	 * @param aCaption New caption.
	 * @see #getCaption
	 * @see #caption
	 * @see #captionEmpty
	 */
    public void setCaption(String aCaption) {
        caption = aCaption;
		layout();
		repaint();
    }

	/** Gets the current caption.
	 * @return Caption.
	 * @see #setCaption
	 * @see #caption
	 * @see #captionEmpty
	 */
    public String getCaption() {
        return caption;
    }

	/** Sets border width around the component.
	 * @param borderWidth New value of margin.
	 * @see #margin
	 * @see #getBorderWidth
	 */
    public void setBorderWidth(int borderWidth) {
        margin = borderWidth;
		layout();
		repaint();
    }

	/** Gets border width around the component.
	 * @see #margin
	 * @see #setBorderWidth
	 */
    public int getBorderWidth() {
        return(margin);
    }

	/** Sets width of the space between the inner and outer borders
	 * @param bevelWidth New value of bevelWidth.
	 * @see #bevelWidth
	 * @see #getBevelWidth
	 */
    public void setBevelWidth(int aBevelWidth) {
        bevelWidth = aBevelWidth;
		layout();
		repaint();
    }

	/** Gets the width of the space between the inner and outer borders
	 * @see #bevelWidth
	 * @see #setBevelWidth
	 */
    public int getBevelWidth() {
        return(bevelWidth);
    }

	/** Sets the border color.
	 * @param col New value of marginColor.
	 * @see #marginColor
	 * @see #defaultBorderColor
	 * @see #getBorderColor
	 */
    public void setBorderColor(Color col) {
        marginColor = col;
		repaint();
    }

	/** Gets the border color.
	 * @return Current border color.
	 * @see #marginColor
	 * @see #defaultBorderColor
	 * @see #setBorderColor
	 */
    public Color getBorderColor() {
        return marginColor;
    }

    public void paint(Graphics g) {
  		Dimension s = size();
		Dimension capdim = getCaptionDim();
		Insets ins = insets();
		int top = 0;
		if (!captionEmpty()) {
			top = bevelWidth;
			if (bevelInner != BV_NONE)
				top += margin;
			if (bevelOuter != BV_NONE)
				top += margin;
			top = (capdim.height - top) / 2;
		}

		int left = 0;
		int w = s.width - 1;
		int h = s.height - 1;
		if (capdim.height != 0)
			h -= top;
		boolean raised;
		
		g.setColor(marginColor);
		for (int tmp = 0; tmp <= 1; tmp++) {
			int tmp2 = bevelOuter;
			if (tmp == 1)
				tmp2 = bevelInner;
			raised = false;
			if (tmp2 == BV_NONE)
				continue;
			else if (tmp2 == BV_RAISED)
				raised = true;
			if (tmp == 1) {
			    left += bevelWidth;
			    top += bevelWidth;
			    w -= 2 * bevelWidth;
			    h -= 2 * bevelWidth;
			}
			for(int i = 0; i < margin; i++) {
			    g.draw3DRect(left, top, w, h, raised);
			    left++;
			    top++;
			    w -= 2;
			    h -= 2;
			}
		}
		if (!captionEmpty()) {
			left = insets().left;
			g.setColor(getBackground());
			g.fillRect(left, 0, capdim.width+8, Math.max(capdim.height, top));
			g.setColor(getForeground());
			g.drawString(caption, left + 4, capdim.height);
		}
    }
	
	/** Gets the dimension of the caption written in the current font.
	 * @return the dimension of the caption.
	 * @see #caption
	 */
	protected Dimension getCaptionDim() {
		if (captionEmpty())
			return new Dimension(0,0);
		Font font = getFont();
		FontMetrics f = Toolkit.getDefaultToolkit().getFontMetrics(
				font==null ? new Font("Helvetica", Font.PLAIN, 12) : font);
		return new Dimension(f.stringWidth(caption), f.getHeight()*2/3);
	}

    public Dimension minimumSize() {
		return countSize(super.minimumSize());
    }

    public Dimension preferredSize() {
		return countSize(super.preferredSize());
    }

	/** Return whether the caption is empty.
	 * @return true if the caption is empty otherwise false.
	 * @see #caption
	 * @see #getCaption
	 * @see #setCaption
	 */
	public boolean captionEmpty() {
		if (caption == null)
			return true;
		if (caption.length() == 0)
			return true;
		return false;
	}

	private Dimension countSize(Dimension d) {
		int x = insets().left + insets().right;
		int y = insets().top + insets().bottom;
		if (!captionEmpty()) {
			Dimension dcap = getCaptionDim();
			x += dcap.width + 8;
			int y2 = dcap.height + 4 + insets().bottom;
			if (y2 > y)
				y = y2;
		}
		if (d.width < x)
			d.width = x;
		if (d.height < y)
			d.height = y;
		return d;
	}
}
