/*
 * Xelfi
 */

package xelfi.awt;

import java.awt.*;

/**
 * A class that produces a Spin Edit.
 *
 * @version 2.1 1 Mar 1997
 * @author  Petr Hamernik
 */
public class SpinEdit extends Panel {
	private SpinButton spin = null;
	private TextField edit = null;

	/** If propagate is set to true all events are propagate and don't
	 *  catch. This flag is used for xawt component during design time.
	 */
	public boolean propagate = false;
	
	/** Create new SpinEdit with all default values. */
	public SpinEdit() {
        setLayout(new SpinEditLayout());
		
		spin = new SpinButton();
		spin.setOrientation(false);
		add("S", spin);
		edit = new TextField();
		add("E", edit);
	}

	/** Sets the border color.
	 * @param col New value of marginColor.
	 * @see SpinButton#marginColor
	 * @see SpinButton#defaultBorderColor
	 * @see #getBorderColor
	 */
    public void setBorderColor(Color col) {
		spin.setBorderColor(col);
    }

	/** Gets the border color.
	 * @return Current border color.
	 * @see SpinButton#marginColor
	 * @see SpinButton#defaultBorderColor
	 * @see #setBorderColor
	 */
    public Color getBorderColor() {
        return spin.getBorderColor();
    }

	/** Sets border width around the component.
	 * @param borderWidth New value of margin.
	 * @see SpinButton#margin
	 * @see #getBorderWidth
	 */
    public void setBorderWidth(int borderWidth) {
		spin.setBorderWidth(borderWidth);
    }

	/** Gets border width around the component.
	 * @see SpinButton#margin
	 * @see #setBorderWidth
	 */
    public int getBorderWidth() {
		return spin.getBorderWidth();
    }

	/** Sets a minimum of the range of the SpinEdit. If value
	 *  or maximum fall out of acceptable values they are adjusted.
	 * @param aMin New minimum.
	 * @see #getMinimum
	 */
	public void setMinimum(int aMin) {
		spin.setMinimum(aMin);
		updateEdit();
	}

	/** Gets the current minimum of the range of SpinEdit.
	 * @return Minimum.
	 * @see #setMinimum
	 */
	public int getMinimum() {
		return spin.getMinimum();
	}
	
	/** Sets a maximum of the range of the SpinEdit. If value
	 *  or minimum fall out of acceptable values they are adjusted.
	 * @param aMax New maximum.
	 * @see #getMinimum
	 */
	public void setMaximum(int aMax) {
		spin.setMaximum(aMax);
		updateEdit();
	}

	/** Gets the current maximum of the range of SpinEdit.
	 * @return Maximum.
	 * @see #setMaximum
	 */
	public int getMaximum() {
		return spin.getMaximum();
	}

	/** Sets a new value of the SpinEdit. If value is outside
	 *  the ranges it is set to nearest acceptable value.
	 * @param aValue New value.
	 * @see #getValue
	 */
	public void setValue(int aValue) {
		spin.setValue(aValue);
		updateEdit();
	}

	/** Gets the current value of the SpinEdit.
	 * @return Value.
	 * @see #setValue
	 */
	public int getValue() {
		return spin.getValue();
	}

	/** Sets a new step of the SpinEdit.
	 * @param aStep New step.
	 * @see #getStep
	 */
	public void setStep(int aStep) {
		spin.setStep(aStep);
		updateEdit();
	}

	/** Gets the current step of the SpinEdit.
	 * @return Step.
	 * @see #setStep
	 */
	public int getStep() {
		return spin.getStep();
	}

	/** Sets new value of repeatDelay variable.
	 * @param aDelay New delay.
	 * @see SpinButton#repeatDelay
	 * @see #getDelay
	 */
	public void setDelay(int aDelay) {
		spin.setDelay(aDelay);
	}

	/** Gets the current value of repeatDelay variable.
	 * @return Delay.
	 * @see SpinButton#repeatDelay
	 * @see #setDelay
	 */
	public int getDelay() {
		return spin.getDelay();
	}

	/** Sets new value of repeatRate variable.
	 * @param aRate New rate.
	 * @see SpinButton#repeatRate
	 * @see #getRate
	 */
	public void setRate(int aRate) {
		spin.setRate(aRate);
	}

	/** Gets the current value of rate variable.
	 * @return Rate.
	 * @see SpinButton#repeatRate
	 * @see #setRate
	 */
	public int getRate() {
		return spin.getRate();
	}

	private void updateEdit() {
		edit.setText(new Integer(spin.getValue()).toString());
	}

    public boolean handleEvent(Event event) {
		spin.propagate = propagate;
		if (propagate)
			return false;
		if (event.target == edit) {
			String s = edit.getText();
			int i;
			try {
				i = Integer.parseInt(s);
			}
			catch (NumberFormatException e) {
				return false;
			}
			spin.setValue(i);
			return false;
		}
    	else if (event.id == SpinButton.SPIN_UP) {
			updateEdit();
			return false;
    	}
		else if (event.id == SpinButton.SPIN_DOWN) {
			updateEdit();
			return false;
    	}
    	return super.handleEvent(event);
	}

	public synchronized void setBackground(Color c) {
		super.setBackground(c);
		edit.setBackground(c);
		spin.setBackground(c);
		hide();
		show();
	}

	public synchronized void setForeground(Color c) {
		super.setForeground(c);
		edit.setForeground(c);
		spin.setForeground(c);
		hide();
		show();
	}

	public synchronized void setFont(Font f) {
		super.setFont(f);
		edit.setFont(f);
		hide();
		show();
	}
}
