package xelfi.awt;

import java.awt.*;
import java.util.Vector;

/**
 * Provides an implementation of owner draw list.
 * Items of list must implement XelfiListViewElement interface.
 * Interface of this class is nearly identical to the interface
 * of java.awt.List class.
	Last change:  DF    3 Feb 97    6:20 pm
*/
public class XFListView implements XelfiScrollable {

	private Vector								items;
	private Vector								selectedItems;
	private boolean								multipleSelections;

	// indent variables
	private int										xIndent;
	private int										yIndent;
	private int										spaceBetween;
	// upper left point of the visible area
	private int										leftX;
	private int										upperY;
	// scrollable owner object
	private XelfiScrollableOwner	scrollableOwner;

	/**
	 * Creates a new scrolling list initialized with 
	 * no visible Lines or multiple selections.
	*/
  public XFListView () {
		this(false);
	}

	/**
	 * Creates new list with multiple selections 
	 * enabled/disabled as specified in param.
	*/
  public XFListView (boolean multipleSelections) {
		this.multipleSelections = multipleSelections;
		initVariables();
	}

  /**
	 * Adds the specified item to the end of scrolling list.
	 * Only reference to the item is added to the list.
	*/
	public void addItem (XelfiListViewElement item) {
		items.addElement((Object)item);
	}

	/**
	 * Adds the specified item to the specified index.
	 * Indexes are counted from 0 base.
	 * Only reference to the item is added to the list.
	*/
	public void addItem (XelfiListViewElement item, int index) {
		items.insertElementAt((Object)item, index);
	}

	/**
	 * Returns true if this list allows multiple selections.
	*/
	public boolean allowsMultipleSelections () {
		return multipleSelections;
	}

	/**
	 * Clears the list.
	*/
	public void clear () {
		items.removeAllElements();
		selectedItems.removeAllElements();
	}

	/**
	 * Returns the number of items in the list.
	*/
	public int countItems () {
		return items.size();
	}

	/**
	 * Returns the number of selected items in the list.
	*/
	public int countSelectedItems () {
		return selectedItems.size();
	}

	/**
	 * Delete an item from the list.<br>
	 * Item is specified by its index, 
	 * counted from 0 for first item in the list.
	 * Returns false if specified item couldn't be deleted;
	 * true otherwise.
	*/
  public boolean delItem (int index) {
		if (index<0 || index>=countItems()) return false;
		Object item = items.elementAt(index);
		selectedItems.removeElement(item);
		items.removeElement(item);
		return true;
	}

	/**
	 * Deselects the item at the specified index.
	 * Returns false if specified item hasn't been selected;
	 * true otherwise.
	*/
	public boolean deselect (int index) {
		Object item = items.elementAt(index);
		if (item==null) return false;
		return selectedItems.removeElement(item);
	}

	/**
	 * Gets the item associated with the specified index.
	*/
	public XelfiListViewElement getItem (int index) {
		return (XelfiListViewElement)items.elementAt(index);
	}

	/**
	 * Get the selected item on the list or -1 if no item is selected.
	 * If multiple selections is set, returns index of 
	 * first selected item.
	*/
	public int getSelectedIndex () {
		if (selectedItems.size()<=0) return -1;
		return items.indexOf(selectedItems.elementAt(0));
	}

	/**
	 * Returns the selected indexes on the list.
	*/
	public int[] getSelectedIndexes () {
		int selectedCount = countSelectedItems();
		int[] selectedIndexes = new int[selectedCount];
		
		for (int i=0; i<selectedCount; i++) {
			selectedIndexes[i] = items.indexOf(selectedItems.elementAt(i));
		}
		return selectedIndexes;
	}

	/**
	 * Returns the selected item on the list 
	 * or null if no item is selected.
	*/
	public XelfiListViewElement getSelectedItem () {
		if (selectedItems.size()<=0) return null;
    return (XelfiListViewElement)selectedItems.elementAt(0);
	}

	/**
	 * Returns the selected items on the list.
	*/
	public XelfiListViewElement[] getSelectedItems () {
		int selectedCount = countSelectedItems();
		XelfiListViewElement[] selItems = 
					new	XelfiListViewElement[selectedCount];
		for (int i=0; i<selectedCount; i++) {
			selItems[i] = (XelfiListViewElement)selectedItems.elementAt(i);
		}
		return selItems;
	}

	/**
	 * Returns true if the item at the specified index has been selected; 
	 * false otherwise.
	*/
	public boolean isSelected (int index) {
		if (index<0 || index>=countItems()) return false;
		return selectedItems.indexOf(items.elementAt(index))!=-1;
	}

	/**
	 * Replaces the item at the given index.
	*/
	public void replaceItem (XelfiListViewElement item, int index) {
		Object oldItem = items.elementAt(index);
		int selIndex = oldItem==null ? -1 : selectedItems.indexOf(oldItem);
		if (selIndex!=-1)
		  selectedItems.setElementAt((Object)item, selIndex);
		items.setElementAt((Object)item, index);
	}

	/**
	 * Selects the item at the specified index.
	*/
	public boolean select (int index) {
		Object item = items.elementAt(index);
		if (item==null) return false;
		if (!multipleSelections)
			selectedItems.removeAllElements();
		if (!isSelected(index)) {
			selectedItems.addElement(item);
			return true;
		} else
			return false;
	}

	/**
	 * Conditionally selects or unselects item at the specified index.
	*/
	public void select (int index, boolean selectItem) {
		if (selectItem)
			select(index);
		else
			deselect(index);
	}

	/**
	 * Deselects all selected items.
	*/
	public void deselectAll () {
		selectedItems.removeAllElements();
	}

	/**
	 * Enables/disables multiple selections for list.
	*/
	public boolean setMultipleSelections (boolean multiple) {
		if (multipleSelections!=multiple) return false;
		multipleSelections = multiple;
		selectedItems.removeAllElements();
		return true;
	}

	/**
	 * Returns inedx of element which contains specified
	 * point. (relative to getView()).
	 * Returns -1 if no such item has been found.
	*/
	public int getIndexFromPoint (int x, int y) {
		// recalculate to absolute coordinates
		int realX = x+leftX;
		int realY = y+upperY;
		int wholeX = xIndent;
		int wholeY = yIndent;
		// and iterate through the list...
		for (int i=0; i<items.size(); i++) {
			Dimension itemSize = 
						((XelfiListViewElement)items.elementAt(i)).preferredSize();
			Rectangle itemRect = new Rectangle(wholeX, wholeY, 
																				 itemSize.width, itemSize.height);
			if (itemRect.inside(realX, realY))
				return i;
			wholeY += itemSize.height+spaceBetween;
		}
		return -1;
	}

	/**
	 * Returns upper left coordinates of the item at index.
	 * Returns newly created object.
	*/
	public Point getUpperLeft (int index) {
		int wholeX = xIndent;
		int wholeY = yIndent;
		// and iterate through the list...
		for (int i=0; i<index; i++) {
			Dimension itemSize = 
						((XelfiListViewElement)items.elementAt(i)).preferredSize();
			wholeY += itemSize.height+spaceBetween;
		}
		return new Point(wholeX, wholeY);
	}

	/**
	 * Access method. Returns horizontal indent.
	*/
	public int getYIndent () {
		return yIndent; 
	}

	/**
	 * Access method. Returns vertical indent.
	*/
	public int getXIndent () {
		return xIndent;
	}

	/**
	 * Access method. Returns horizontal space between items.
	*/
	public int getSpaceBetween () {
		return spaceBetween;
	}

	/**
	 * Access method. Sets horizontal indent.
	*/
	public void setYIndent (int yIndent) {
		this.yIndent = yIndent;
	}

	/**
	 * Access method. Sets vertical indent.
	*/
	public void setXIndent (int xIndent) {
		this.xIndent = xIndent;
	}

	/**
	 * Access method. Sets horizontal space between items.
	*/
	public void setSpaceBetween (int spaceBetween) {
		this.spaceBetween = spaceBetween;
	}

	// Implementation of the interface XelfiScrollable follows
	/**
	 * Sets the upper left point of the view.
	*/
	public void setView (int x, int y) {
		leftX = x;
		upperY = y;
	}

	/**
	 * Returns upper left corner of the view.
	*/
	public Point getView () {
		return new Point(leftX, upperY);
	}

	/**
	 * Paints specified area, starting from getView() point
	 * with width and height as specified in params.
	*/
	public void paintArea (Graphics g, int width, int height) {
		paintArea(g, leftX, upperY, width, height, false);
	}

	/**
	 * Paints specified area of the list
	*/
	public void paintArea (Graphics g, int x, int y, int width, int height,
													boolean clearBackground) {
		if (clearBackground)
			g.clearRect(x-leftX, y-upperY, width, height);
		// create paint rectangle
		Rectangle paintRect = new Rectangle(x, y, width, height);
		int globalX = xIndent;
		int globalY = yIndent;
		// walk through the list and paint items....
		for (int i=0; i<items.size(); i++) {
			XelfiListViewElement item = (XelfiListViewElement)items.elementAt(i);
			Dimension itemSize = item.preferredSize();
			// check if item is in paint area
			Rectangle itemRect = new Rectangle(globalX, globalY, 
																					itemSize.width, itemSize.height);
      if (paintRect.intersects(itemRect)) {
				// yep, so paint item
				if (isSelected(i))
					item.paintSelected(g, globalX-leftX,
														 globalY+itemSize.height/2-upperY);
				else
					item.paint(g, globalX-leftX,
										 globalY+itemSize.height/2-upperY);
			}
			// move to the next item
			globalY += itemSize.height+spaceBetween;
		}
	}

	/**
	 * Returns visible size.
	 * ListView has no visible area so it returns whole area.
	*/
	public Dimension visibleSize () {
		return wholeSize();
	}

  /**
	 * Returns dimension needed for all scrollable
	 * object screen area to be shown.
	*/
	public Dimension wholeSize () {
		int wholeX = 0;
		int wholeY = 0;
		for (int i=0; i<items.size(); i++) {
			Dimension itemSize = 
						((XelfiListViewElement)items.elementAt(i)).preferredSize();
			if (itemSize.width>wholeX)
				wholeX = itemSize.width;
			wholeY += itemSize.height+spaceBetween;
		}
		// add borders
		wholeX += 2*xIndent;
		wholeY += 2*yIndent-spaceBetween;
		return new Dimension(wholeX, wholeY);
	}

	/**
	 * Sets scrollable owner - object which holds scrollbars
	 * and should be notified when changes in coordinates of
	 * visible area are made.
	*/
	public void setScrollableOwner (XelfiScrollableOwner scrollableOwner) {
		this.scrollableOwner = scrollableOwner;
	}

	/**
	 * Returns scrollable owner - object which usually holds scrollbars
	*/
	public XelfiScrollableOwner getScrollableOwner () {
	  return scrollableOwner;
	}

	/**
	 * Initialization method. Initializes variables.
	 * Should be called only from constructor.
	*/
	private void initVariables () {
		xIndent = 5;
		yIndent = 5;
		spaceBetween = 1;
		items = new Vector(200, 100);
		selectedItems = new Vector(5, 5);
		scrollableOwner = null;
	}

}

