package xelfi.awt;

import java.util.Vector;

import xelfi.util.Debug;

/**
 * Implementation of node of the XFTree tree. Each node can have 
 * multiple children. Supports methods for enumarating children,
 * inserting, removing and retrieving sub-tree size.
 *
 * @version 1.01, 19 Nov 1996
 * @author Dafe
	Last change:  DF   14 Mar 97   12:45 pm
*/
public class XFTreeNode {
	// an object - value which represents node
	Object					data;
	// user object - additional information
	Object					userData;
	// children and parent
	Vector					children;
	XFTreeNode			parent;

	public XFTreeNode (XFTreeNode parent, Object data, Object userData) {
		this.data = data;
		this.parent = parent;
		this.userData = userData;
		children = new Vector(10, 10);
	}

	public XFTreeNode (XFTreeNode parent, Object data) {
		this(parent, data, null);
	}

	/**
	 * Access method.
	*/
	public XFTreeNode getParent () {
		return parent;
	}

	/**
	 * Access method - sets new parent 
	 * (needed for moving items across the tree)
	*/
	public void setParent (XFTreeNode parent) {
		this.parent = parent;
	}

	/**
	 * Access method.
	*/
	public Object getData () {
		return data;
	}

	/**
	 * Access method.
	*/
	public Object getUserData () {
		return userData;
	}

	/**
	 * Access method. Sets additional user object.
	*/
	public void setUserData (Object userData) {
		this.userData = userData;
	}

	/**
	 * Returns count of its direct children
	*/
	public int getChildCount () {
		return children.size();
	}

	/**
	 * Returns count of its children at all sublevels,
	 * e.g. in whole subtree
	*/
	public int getAllChildCount () {
		int result = 0;
		return subTreeSize(result);
	}

	/**
	 * Returns child node at specified index
	*/
	public XFTreeNode getChild (int index) {
		if ((index<0) || (index >= children.size())) return null;
		XFTreeNode node = (XFTreeNode)children.elementAt(index);
		return node;
	}

	/**
	 * Creates and inserts new node after specified child node if it has been
	 * found or as first child if "after" can't be found.
	*/
	public XFTreeNode insert (Object data, XFTreeNode after) {
		int found = findNodeAfter(after);
		XFTreeNode added = new XFTreeNode(this, data);
		children.insertElementAt(added, found);
		return added;
	}

	/**
	 * Inserts new node after specified child node if it has been
	 * found or as first child if "after" can't be found.
	*/
	public XFTreeNode insert (XFTreeNode node, XFTreeNode after) {
		int found = findNodeAfter(after);
		children.insertElementAt(node, found);
		return node;
	}

	/**
	 * Removes specified child node.
	 * Returns false if child cannot be found
	*/
	public boolean remove (XFTreeNode node) {
		return children.removeElement(node);
	}

	/**
	 * Finds node with specified data in subtree
	*/
	public XFTreeNode findNodeWithData (XelfiElementComparer comparer, 
																			Object data) {
		if (comparer.compare(getData(), data)) return this;
		XFTreeNode result;
		for (int i=0; i<getChildCount(); i++) {
			result = getChild(i).findNodeWithData(comparer, data);
			if (result!=null) return result;
		}
		return null;
	}

	/**
	 * Decides whether a node is a child (in whole subtree)
	 * of this node or not.
	*/
	public boolean isChild (XFTreeNode node) {
		for (int i=0; i<getChildCount(); i++) {
			XFTreeNode curChild = getChild(i);
			if (node == curChild) return true;
			if (curChild.isChild(node)) return true;
		}
		return false;
	}

	/**
	 * Finds node after specified, returns its index.
	 * Returns 0 if after has not been found
	*/
	private	int findNodeAfter (XFTreeNode after) {
		int found = 0;
		for (int i=0; i<children.size(); i++) {
			if ((XFTreeNode)children.elementAt(i)==after) {
				found = i+1;
				break;
			}
		}
		return found;
	}

	/**
	 * Performs the work of counting children in subtree below
	 * this element.
	*/
	private int subTreeSize (int count) {
		for (int i=0; i<getChildCount(); i++)
			count += getChild(i).getAllChildCount()+1;
		return count;
	}

}
