
package xelfi.browser.util;
import java.awt.*;
/**
 * Vert layout is used to layout buttons in a panel. It will arrange
 * buttons left to right until no more buttons fit on the same line.
 * Each line may be centered.
 *
 * @version     1.0, 24.2.1997
 * @author  Jan Jancura
 */

public class VertLayout implements LayoutManager {

    /**
     * The left alignment variable.
     */
    public static final int LEFT    = 0;

    /**
     * The center alignment variable.
     */
    public static final int CENTER  = 1;

    /**
     * The right alignment variable.
     */
    public static final int RIGHT   = 2;

    /**
     * The resize alignment variable.
     */
    public static final int RESIZE  = 3;

    /**
     * The top alignment variable.
     */
    public static final int TOP     = 4;

    /**
     * The bottom alignment variable.
     */
    public static final int BOTTOM  = 5;


    int alignH;
    int alignV;
    int hgap;
    int vgap;

    /**
     * Constructs a new Vert Layout with a centered alignments.
     */
    public VertLayout() {
    this(CENTER,CENTER, 5, 5);
    }

    /**
     * Constructs a new Vert Layout with the specified alignments.
     * @param alignH the horizontal alignment value
     * @param alignV the vertical alignment value
     */
    public VertLayout(int alignH,int alignV) {
    this(alignH,alignV, 5, 5);
    }

    /**
     * Constructs a new Vert Layout with the specified alignment and gap
     * values.
     * @param alignH the horizontal alignment value
     * @param alignV the vertical alignment value
     * @param hgap the horizontal gap variable
     * @param vgap the vertical gap variable
     */
    public VertLayout(int alignH,int alignV, int hgap, int vgap) {
    this.alignH = alignH;
    this.alignV = alignV;
    this.hgap = hgap;
    this.vgap = vgap;
    }

    /**
     * Adds the specified component to the layout. Not used by this class.
     * @param name the name of the component
     * @param comp the the component to be added
     */
    public void addLayoutComponent(String name, Component comp) {
    }

    /**
     * Removes the specified component from the layout. Not used by
     * this class.
     * @param comp the component to remove
     */
    public void removeLayoutComponent(Component comp) {
    }

    /**
     * Returns the preferred dimensions for this layout given the components
     * in the specified target container.
     * @param target the component which needs to be laid out
     * @see Container
     * @see #minimumLayoutSize
     */
    public Dimension preferredLayoutSize(Container target) {
    Dimension dim = new Dimension(0, 0);
    int nmembers = target.countComponents();

    for (int i = 0 ; i < nmembers ; i++) {
        Component m = target.getComponent(i);
        if (m.isVisible()) {
    	Dimension d = m.preferredSize();
    	dim.height = Math.max(dim.height, d.height);
    	if (i > 0) {
    	    dim.width += hgap;
    	}
    	dim.width += d.width;
        }
    }
    Insets insets = target.insets();
    dim.width += insets.left + insets.right + hgap*2;
    dim.height += insets.top + insets.bottom + vgap*2;
    return dim;
    }

    /**
     * Returns the minimum dimensions needed to layout the components
     * contained in the specified target container.
     * @param target the component which needs to be laid out
     * @see #preferredLayoutSize
     */
    public Dimension minimumLayoutSize(Container target) {
    Dimension dim = new Dimension(0, 0);
    int nmembers = target.countComponents();

    for (int i = 0 ; i < nmembers ; i++) {
        Component m = target.getComponent(i);
        if (m.isVisible()) {
    	Dimension d = m.minimumSize();
    	dim.height = Math.max(dim.height, d.height);
    	if (i > 0) {
    	    dim.width += hgap;
    	}
    	dim.width += d.width;
        }
    }
    Insets insets = target.insets();
    dim.width += insets.left + insets.right + hgap*2;
    dim.height += insets.top + insets.bottom + vgap*2;
    return dim;
    }

    /**
     * Centers the elements in the specified row, if there is any slack.
     * @param target the component which needs to be moved
     * @param x the x coordinate
     * @param y the y coordinate
     * @param width the width dimensions
     * @param height the height dimensions
     * @param rowStart the beginning of the row
     * @param rowEnd the the ending of the row
     */
    private void moveComponents(Container target, int x, int y, int width, int height, int rowStart, int rowEnd)
    {
        switch (alignH)
        {
        case LEFT:
            break;
        case CENTER:
            x += width / 2;
            break;
        case RIGHT:
            x += width;
            break;
        }
        for (int i = rowStart ; i < rowEnd ; i++)
        {
            Component m = target.getComponent(i);
            if (m.isVisible())
            {
                switch (alignV)
                {
                    case TOP:
                        m.move( x, y);
                        break;
                    case CENTER:
                        m.move( x, y + (height - m.size().height)/2);
                        break;
                    case BOTTOM:
                        m.move( x, y + height - m.size().height);
                        break;
                    case RESIZE:
                        m.reshape( x, y, m.size().width, height);
                        break;
                }
    	        x += hgap + m.size().width;
            }
        }
    }

    /**
     * Lays out the container. This method will actually reshape the
     * components in the target in order to satisfy the constraints of
     * the BorderLayout object.
     * @param target the specified component being laid out.
     * @see Container
     */
    public void layoutContainer(Container target) {
    Insets insets = target.insets();
    int maxwidth = target.size().width - (insets.left + insets.right + hgap*2);
    int nmembers = target.countComponents();
    int x = 0, y = insets.top + vgap;
    int rowh = 0, start = 0;

    for (int i = 0 ; i < nmembers ; i++) {
        Component m = target.getComponent(i);
        if (m.isVisible()) {
    	Dimension d = m.preferredSize();
    	m.resize(d.width, d.height);

    	if ((x == 0) || ((x + d.width) <= maxwidth)) {
    	    if (x > 0) {
    	   x += hgap;
    	    }
    	    x += d.width;
    	    rowh = Math.max(rowh, d.height);
    	} else {
    	    moveComponents(target, insets.left + hgap, y, maxwidth - x, rowh, start, i);
    	    x = d.width;
    	    y += vgap + rowh;
    	    rowh = d.height;
    	    start = i;
    	}
        }
    }
    moveComponents(target, insets.left + hgap, y, maxwidth - x, rowh, start, nmembers);
    }

    /**
     * Returns the String representation of this VertLayout's values.
     */
    public String toString()
    {
        String str = "";
        switch( alignH )
        {
        case LEFT:    str = ",alignH=left"; break;
        case CENTER:  str = ",alignH=center"; break;
        case RIGHT:   str = ",alignH=right"; break;
        }
        switch( alignV )
        {
        case TOP:     str = str + ",alignV=top";break;
        case CENTER:  str = str + ",alignV=center";break;
        case BOTTOM:  str = str + ",alignV=bottom";break;
        case RESIZE:  str = str + ",alignV=resize";break;
        }
        return getClass().getName() + "[hgap=" + hgap + ",vgap=" + vgap +
            str + "]";
    }
}

