/** ErrorColorer class
* @author: Filip Dvorak
* @version: 1.0
*/

package xelfi.compiler;

import java.awt.*;
import xelfi.top.XFColor;
import xelfi.top.XFColorCallback;
import xelfi.editor.*;

  /** This class is "inserted" into JavaEditorFrame (as it implements BackgroundCallback) and highlights
    error lines. One instance can highlight source for only one class.
  */
public class ErrorColorer implements BackgroundCallback, ActionController, XFColorCallback
{
  protected ErrorDescription highlightedDesc;

    /** This can be set only after Description was set. Position != null <==> bc != null <==> hlBlock != null. */
  private Position highlightedPosition;

    // Handled by ErrorColorerEvidence
  int noReferences;

    /** local caching of color values */
  protected static Color foregroundColor;
  protected static Color backgroundColor;

  private static boolean firstInstance = true;

    /** Creates empty error colorer. */
  ErrorColorer()
  {
    highlightedDesc = null;
    highlightedPosition = null;
    initColorCallback();
  }

  private static synchronized void initColorCallback()
  {
    if(firstInstance)
    {
      firstInstance = false;
      XFColor.addCallback(new ErrorColorer());  // this new class serves only as XFColorCallback
    }
  }

    /** Called by XFColor whan color settings has changed. Copies new color values from XFColor.*/
  public void colorsChanged()
  {
    foregroundColor = XFColor.getFgColor(XFColor.ErrorLine);
    backgroundColor = XFColor.getBgColor(XFColor.ErrorLine);
  }

    /** BackgroundCallback implementation returns foreground color. */
  public Color getForeground()
  {
    return foregroundColor;
  }

    /** BackgroundCallback implementation returns background color for position or null if the position
      is not on highlighted error line. 
      @param p a position in the source text
      @return color for the line determinet by the position.
      */
  public Color getBackground(Position p)
  {
    if(highlightedDesc == null) return null;

    try
    {
      if(highlightedPosition.sameLine(p)) return backgroundColor;
      else return null;
    }
    catch(NullPointerException e)
    {
        // we determine Position and install ActionController now
        // Kdyz se pak Editor zavre, tak highlightedPosition  bude porad drzet Document v pameti.
        // dalo by se to vyresit documentFreeNotification
      highlightedPosition = highlightedDesc.getPosition();
      highlightedPosition.getDocument().addActionController(this);

      if(highlightedPosition.sameLine(p)) return backgroundColor;
      else return null;
    }
  }

  ErrorDescription getHighlight()
  {
    return highlightedDesc;
  }

    // cleans color on the old highlighted line, and sets new hl line
    // new hl line is not redrawn. 
  void setHighlight(ErrorDescription desc)
  {
    highlightedDesc = desc;
    if(highlightedPosition != null)
    {
      Block oldHl = Block.line(highlightedPosition);
      if(desc != null) highlightedPosition = desc.getPosition();
      else // only remove highlighted line
      {
          highlightedPosition.getDocument().removeActionController(this);
          highlightedPosition = null;
      }
      oldHl.redraw();      // redraw old error line
    }
  }

  /* This method is called before every insert to any position
  * For multiple inserts (of arrays or Strings) is called only
  * once at begining.
  *
  *
  * @param p the position method insert will insert to
  * @param info optional info sent to insert or null if nothing has been sent
  * @exception NotAllowedException if this insert is not allowed
  */
  public void allowInsert (Position p, Object info) throws NotAllowedException
  {}

  /* This method is called before every delete of single character at any position.
  *
  * @param p the position method delete will delete at
  * @param info optional info sent to delete
  * @exception NotAllowedException if this delete is not allowed
  */
  public void allowDelete (Position p, Object info) throws NotAllowedException
  {}

  /* This method is called before every delete of a block (more characters).
  * @param from the first position to delete
  * @param to the last (excluded) position
  * @param info the info
  * @exception NotAllowedException if this delete is not allowed
  */
  public void allowMultiDelete (Position from, Position to, Object info) throws NotAllowedException
  {}

  /* Notifies that an insert has been done at position p.
  * @param n the begin of inserted text
  * @param p the end of inserted text
  */
  public void notifyInsert (Position n, Position p)
  {
    if(p.sameLine(highlightedPosition)) setHighlight(null);
  }

  /* Notifies a delete
  * @param p the position where a delete occures
  * @param s the string with characters that has been deleted
  */
  public void notifyDelete (Position p, String s)
  {
    if(p.sameLine(highlightedPosition)) setHighlight(null);
  }

  /* Handle event allows sending some predefined messages to position
  * and by throw the document to ActionController
  * @param p the position a message is sent to
  * @param e the event (the same as in java.awt)
  * @return true to signal successful handling
  * @see java.awt.Event
  */
  public boolean handleEvent(Position p, Event e)
  {
//    if(e.id == Constants.CHANGED_CURSOR)
    return false;
  }
}

