/**
 * Watches
 *
 * @author Roman Blazevic
 * @version 970208
 */

package xelfi.debugger;

import java.awt.*;
import xelfi.util.EnhProperties;

/**
 * This is a document class which stores information about all watched variables.
 */

public class Watches
{
	private Debugger debugger;

	private Watch data[];
	private WatchesView view;

	/**
	 * Watches constructor.
	 */
	Watches(Debugger debugger)
	{
		this.debugger = debugger;

		data = new Watch[0];
		view = new WatchesView(this);
	}

	/**
	 * Returns a number of watches.
	 */
	int getNumberOfWatches()
	{
		return data.length;
	}

	/**
	 * Returns the watch with the specified index.
	 */
	Watch getWatch(int index) throws ArrayIndexOutOfBoundsException
	{
		return data[index];
	}

	/**
	 * Return the view belonging to this document.
	 */
	public WatchesView getView()
	{
		return view;
	}

	/**
	 * Called by the WatchesView class after its instance has been shown.
	 */
	void watchesViewShown()
	{
		updateContent();
	}

	/**
	 * It restores current values of all watches and updates the view on the document.
	 */
	void updateContent()
	{
		for(int i = 0; i < data.length; i++)
			data[i].setCurrentValue(debugger.getCurrentValue(data[i].getName()));
		view.update(WatchesView.HINT_COMPLETE_UPDATE, -1);
	}

	/**
	 * Gets the (watches) properties.
	 */
	void getProperties(EnhProperties properties)
	{
		int numberOfWatches = properties.getIntProperty("NumberOfWatches", 0);

		data = new Watch[0];
		for(int i = 0; i < numberOfWatches; i++)
		{
			String watchName = properties.getProperty("Watch"+(i+1), "");

			if (watchName.compareTo("") != 0)
				addWatch(new Watch(watchName));
		}

		updateContent();
	}

	/**
	 * Puts the (watches) properties.
	 */
	void putProperties(EnhProperties properties)
	{
		int numberOfWatches = getNumberOfWatches();

		for(int i = 0; i < numberOfWatches; i++)
			properties.remove("Watches"+(i+1));

		numberOfWatches = getNumberOfWatches();
		properties.setIntProperty("NumberOfWatches", numberOfWatches);

		for(int i = 0; i < numberOfWatches; i++)
			properties.put("Watch"+(i+1), getWatch(i).getName());
	}

	/**
	 * Inserts a new watch on the position with the specified index.
	 */
	void insertWatch(int index, Watch watch) throws ArrayIndexOutOfBoundsException
	{
		int numberOfWatches = getNumberOfWatches();

		for(int i = 0; i < numberOfWatches; i++)
			if (getWatch(i).getName().equals(watch.getName()))
				return;

		int newSize = data.length+1;
		Watch newData[] = new Watch[newSize];

		for(int i = 0; i < index; i++)
			newData[i] = data[i];
		newData[index] = watch;
		for(int i = index+1; i < newSize; i++)
			newData[i] = data[i-1];

		data = newData;

		watch.setCurrentValue(debugger.getCurrentValue(watch.getName()));
		view.update(WatchesView.HINT_INSERTED, index);
	}

	/**
	 * Add a new watch to the end.
	 */
	void addWatch(Watch watch)
	{
		int numberOfWatches = getNumberOfWatches();

		for(int i = 0; i < numberOfWatches; i++)
			if (getWatch(i).getName().equals(watch.getName()))
				return;

		Watch newData[] = new Watch[numberOfWatches+1];

		for(int i = 0; i < numberOfWatches; i++)
			newData[i] = data[i];
		newData[numberOfWatches] = watch;

		data = newData;

		watch.setCurrentValue(debugger.getCurrentValue(watch.getName()));
		view.update(WatchesView.HINT_INSERTED, numberOfWatches);
	}

	/**
	 * Remove the breakpoint from the specified index.
	 */
	void removeWatch(int index) throws ArrayIndexOutOfBoundsException, NegativeArraySizeException
	{
		int newSize = getNumberOfWatches()-1;
		Watch newData[] = new Watch[newSize];

		for(int i = 0; i < index; i++)
			newData[i] = data[i];
		for(int i = index; i < newSize; i++)
			newData[i] = data[i+1];

		data = newData;

		view.update(WatchesView.HINT_REMOVED, index);
	}

	/**
	 * Replaces the watch on the specified index.
	 */
	void replaceWatch(int index, Watch watch) throws ArrayIndexOutOfBoundsException
	{
		int numberOfWatches = getNumberOfWatches();

		for(int i = 0; i < numberOfWatches; i++)
			if (getWatch(i).getName().equals(watch.getName()))
				return;

		data[index] = watch;

		watch.setCurrentValue(debugger.getCurrentValue(watch.getName()));
		view.update(WatchesView.HINT_CHANGED, index);
	}
}
