/**
 * WatchesView
 *
 * @author Roman Blazevic
 * @version 970208
 */

package xelfi.debugger;

import java.awt.*;
import xelfi.top.TopLevel;
import xelfi.top.TopDialog;

/**
  This is a view on data stored in the document class Watches.
 */

public class WatchesView extends TopDialog
{
	private Watches document;
	private List list;
	private WatchDialog dlg;

	static final int HINT_COMPLETE_UPDATE = 0;
	static final int HINT_INSERTED = 1;
	static final int HINT_REMOVED = 2;
	static final int HINT_CHANGED = 3;

	static final int KEY_ENTER = 10;
	static final int KEY_DELETE = 127;

	private static final int PROCESSING_NO_DIALOG = 0;
	private static final int PROCESSING_INSERT_DIALOG = 1;
	private static final int PROCESSING_CHANGE_DIALOG = 2;

	private int state;
	private int lineStamp;

	/**
	 * WatchesView constructor.
	 *
	 * watches - document for this view
	 */
	WatchesView(Watches watches)
	{
		super(TopLevel.getMenu(), false);
		setTitle("Watches");
		//super("Watches");
		setBackground(Color.lightGray);

		state = PROCESSING_NO_DIALOG;

		document = watches;
		list = new List();
		add("Center", list);
		dlg = null;
	}

	/**
	 * This method is called when the "Add watch" dialog must be processed.
	 */
	void addWatch()
	{
		Event event = new Event(this, 0, Event.KEY_PRESS, 0, 0, 9, 0);

		handleEvent(event);
	}

	/**
	 * My handleEvent() implementation.
	 */
	public boolean handleEvent(Event event)
	{
		Event eventCopy = event;
		int selectedIndex = list.getSelectedIndex();

		if (event.id == Event.ACTION_EVENT)
		{
			eventCopy.id = Event.KEY_PRESS;

			if (selectedIndex == list.countItems()-1)
				eventCopy.key = 9; // Ctrl+I - insert watch
			else
				eventCopy.key = KEY_ENTER;
		}

		if (event.id == Event.KEY_PRESS && event.key == KEY_ENTER)
			if (selectedIndex == list.countItems()-1)
				eventCopy.key = 9; // Ctrl+I - insert watch

		if (eventCopy.id == Event.KEY_PRESS)
		{
			if (dlg == null) // dialog is already shown
			{
				switch (eventCopy.key)
				{
					case 9: // Ctrl+I - insert watch
						dlg = new WatchDialog(this, "Insert a variable name", "");
						state = PROCESSING_INSERT_DIALOG;
						lineStamp = list.getSelectedIndex();
						dlg.show();
						break;
					case KEY_DELETE:
						if (selectedIndex != -1 && selectedIndex != list.countItems()-1)
							document.removeWatch(selectedIndex);
						break;
					case KEY_ENTER:
						if (selectedIndex != -1 && selectedIndex != list.countItems()-1)
						{
							dlg = new WatchDialog(this, "Change a variable name", document.getWatch(selectedIndex).getName());
							state = PROCESSING_CHANGE_DIALOG;
							lineStamp = list.getSelectedIndex();
							dlg.show();
						}
						break;
					default:
						return super.handleEvent(event);
				}
				return true;
			}
		}
		return super.handleEvent(event);
	}

	/**
	 * My action() implementation.
	 * This method processes the result sent from the instance of the BreakpointDialogResult.
	 */
	public boolean action(Event event, Object what)
	{
		if (event == null)
		{
			WatchDialogResult result;

			result = (WatchDialogResult)what;

			if (state == PROCESSING_INSERT_DIALOG)
				if (result.returnCode == WatchDialog.ID_OK)
					if (lineStamp != -1)
						document.insertWatch(lineStamp, new Watch(result.name));
					else
						document.addWatch(new Watch(result.name));

			if (state == PROCESSING_CHANGE_DIALOG)
				if (result.returnCode == WatchDialog.ID_OK)
					if (lineStamp != -1)
						document.replaceWatch(lineStamp, new Watch(result.name));
					else
						document.replaceWatch(list.countItems()-1, new Watch(result.name));

			state = PROCESSING_NO_DIALOG;
			dlg = null;
			return true;
		}
		return super.action(event, what);
	}

	/**
	 * This method is called when the content of the view must be
	 * updated.
	 */
	void update(int hint, int index)
	{
		if (!isVisible())
			return;

		int numberOfWatches = document.getNumberOfWatches();

		switch (hint)
		{
			case HINT_COMPLETE_UPDATE:
				if (numberOfWatches != list.countItems()-1)
				{
					list.clear();
					for(int i = 0; i < numberOfWatches; i++)
						list.addItem(document.getWatch(i).toString());
					list.addItem(""); // last line always contains string ""
				}
				else // equals
					for(int i = 0; i < numberOfWatches; i++)
					{
						String line = document.getWatch(i).toString();

						if (!line.equals(list.getItem(i)))
							list.replaceItem(line, i);
					}

				int indexOfSelected = list.getSelectedIndex();
				if (indexOfSelected != -1)
					list.deselect(indexOfSelected);

				break;
			case HINT_INSERTED:
				list.delItems(index, list.countItems()-1);
				for(int i = index; i < numberOfWatches; i++)
					list.addItem(document.getWatch(i).toString());
				list.addItem(""); // last line always contains string ""
				list.select(index+1);
				break;
			case HINT_REMOVED:
				list.delItems(index, list.countItems()-1);
				for(int i = index; i < numberOfWatches; i++)
					list.addItem(document.getWatch(i).toString());
				list.addItem(""); // last line always contains string ""
				list.select(index);
				break;
			case HINT_CHANGED:
				list.replaceItem(document.getWatch(index).toString(), index);
				list.select(index);
				break;
			default:
				System.out.println("unsupported hint");
				break;
		}
	}

	/**
	 * My show() implementation.
	 */
	public void show()
	{
		super.show();
		list.requestFocus();
		document.watchesViewShown();
	}
}
