/**
 * Xelfi
 */

package xelfi.design.forms;

import java.awt.*;
import xelfi.design.components.*;
import xelfi.design.components.xawt.XAWTPanel;
import xelfi.design.util.*;

/**
 * Absolute layout is used to layout Xelfi design components at absolute positions.
 * It is intended to use during Xelfi design time only 
 * (thus all components in the container must be XelfiVisualComponent components).
 * You may ask why I had to wrote layout manager for absolute positioning.
 * Well, it is because that with layout set to null method validate() does nothing and
 * I have not found a way to repaint component after its look changed. It is also
 * easier to manage components at absolute positions by separate class moreover
 * when having special methods getPosition() and getSize() in component.
 *
 * @version 	1.16, 27 Feb 1997
 * @author 	Ian Formanek
 */

public class AbsoluteLayout implements LayoutManager {

	/**
	 * Constructs a new Absolute Layout.
	 */
	public AbsoluteLayout() {
	}

	/**
	 * Adds the specified component to the layout. Not used by this class.
	 * @param name the name of the component
	 * @param comp the the component to be added
	 */
	public void addLayoutComponent(String name, Component comp) {
	}

	/**
	 * Removes the specified component from the layout. Not used by
	 * this class.  
	 * @param comp the component to remove
	 */
	public void removeLayoutComponent(Component comp) {
	}

	/**
	 * Returns the preferred dimensions for this layout given the components
	 * in the specified target container.
	 * @param target the component which needs to be laid out
	 * @see Container
	 * @see #minimumLayoutSize
	 */
	public Dimension preferredLayoutSize(Container target) {
		Dimension dim = new Dimension(0, 0);
		int nmembers = target.countComponents();

		for (int i = 0 ; i < nmembers ; i++) {
			Component m = target.getComponent(i);
			
			if ((m.isVisible()) && (m instanceof XelfiVisualComponent)){
				Dimension d = m.size();
				Point p = m.location();
				
				if (dim.height < d.height+p.y) dim.height = d.height+p.y;
				if (dim.width < d.width+p.x) dim.width = d.width+p.x;
			}
		}
		Insets insets = target.insets();
		dim.width += insets.left + insets.right;
		dim.height += insets.top + insets.bottom;
		return dim;
	}

	/**
	 * Returns the minimum dimensions needed to layout the components
	 * contained in the specified target container.
	 * @param target the component which needs to be laid out 
	 * @see #preferredLayoutSize
	 */
	public Dimension minimumLayoutSize(Container target) {
		Dimension dim = new Dimension(0, 0);
		int nmembers = target.countComponents();

		for (int i = 0 ; i < nmembers ; i++) {
			Component m = target.getComponent(i);
			if (m.isVisible()) {
				Dimension d = m.minimumSize();
				dim.height += d.height;
				dim.width += d.width;
			}
		}
		Insets insets = target.insets();
		dim.width += insets.left + insets.right;
		dim.height += insets.top + insets.bottom;
		return dim;
	}

	/**
	 * Lays out the container. This method will actually reshape the
	 * XelfiVisualComponents in the target according to their real position and size.
	 * @param target the specified container being laid out.
	 * @see Container
	 */
	public void layoutContainer(Container target) {
		Insets insets = target.insets();
		int nmembers = target.countComponents();

		for (int i = 0 ; i < nmembers ; i++) {
			Component m = target.getComponent(i);
			if ((m.isVisible()) && (m instanceof XelfiVisualComponent)) {
				XelfiVisualComponent xvcm = (XelfiVisualComponent)m;
				Dimension d = xvcm.getDesignSize();
//				Dimension d = ((Component)xvcm).preferredSize();
				Point p = xvcm.getPosition();
				if (xvcm.getSelection() != XFDesignConstants.SEL_UNSELECT) {
					p.x -= XFDesignOptions.selectionSize;
					p.y -= XFDesignOptions.selectionSize;
			}
			m.reshape(insets.left+p.x,insets.top+p.y,d.width, d.height);
			}
		}
	}
    
	/**
	 * Returns the String representation of this layout's values.
	 */
	public String toString() {
		return getClass().getName();
	}
}
