package xelfi.design.inspector;

import java.awt.*;
import java.util.Vector;

import xelfi.design.forms.*;
import xelfi.awt.*;
import xelfi.design.util.*;
import xelfi.top.*;

/**
 * XFComponentInspector is a window for viewing/setting component
 * properties, events and layouts.
 *
 * @author Dafe, Ian Formanek
 * version 1.26, 26 Nov 1996 - inicialization bug fixed
 * version 1.27, 27 Nov 1996 - supports read only properties
 * version 1.28, 28 Nov 1996 - supports strict choices
 * version 1.29, 2 Dec 1996 - refresh of other properties ready,
 *                            added PVT_INPUT_CHAR,
 *                            Escape key now work (escapes editing)
 * version 1.30, 3 Dec 1996 - component selecting choice works
 *
 *
 * @version 1.30, 3 Dec 1996
   Last change:  DF   16 Apr 97   11:08 am
*/
public class XFComponentInspector
     extends TopDialog implements XelfiComponentInspector {

   // choice   to select from inspected components
   private XFSelectComponentPanel  selectPanel = null;
   // main properties panel
   private XFPropertyPanel         propertyPanel = null;
   // attached inspectable source, which gives selected components etc.
   private XelfiDesignInspectable  inspectable = null;
   // inner class to work with multiple (common) properties like with one
   private XFCommonProperties      commonProperties = null;
   // page switcher
   private TabControl              pageTab = null;
//   private   int                       page = XFDesignConstants.CI_PG_PROPERTIES;
   // selecting in done in different thread
   private SelectionThread           selectionThread = null;
   private PageThread              pageThread = null;
   // old selection
  private Vector                   oldSelection = null;


   public XFComponentInspector(Frame parent) {
//     super(parent, "Component Inspector", false);
     super(parent, false);

     setTitle("Component Inspector");
     // sets good background
    setBackground(Color.lightGray);
//     setFont(new Font("Helvetica",Font.PLAIN,11));

     // creates sub-panels
     selectPanel = new XFSelectComponentPanel(this);
     propertyPanel = new XFPropertyPanel(commonProperties);
     pageTab = new TabControl();

     add("North", selectPanel);
     add("Center", propertyPanel);
     add("South", pageTab);

     // set page names in tabControl
     pageTab.addTab("Properties");
     pageTab.addTab("Events");
     pageTab.addTab("Layout");
     // set the direction
     pageTab.setDirection(false);
   }

   /**
     *   Notification method from XelfiComponentInspector interface.
     * This says - hey, selected components have changed - so do
     * something with it.
   */
   public void selectionChanged () {
     if (inspectable==null) {
      commonProperties = null;
      // clear all and disable select panel
      selectPanel.empty();
      selectPanel.changeContent();
      propertyPanel.empty();
      propertyPanel.repaint();
      return;
     }
     // wait until threads can be STOPPED and
     // thrown to the binary space :-))
     selectPanel.waitUntilStoppable();
    if (pageThread!=null) pageThread.stop();
     if (selectionThread!=null) selectionThread.stop();
     // starts new thread, which performs the real work of
     // changing the selection
     selectionThread = new SelectionThread(this);
     selectionThread.start();

/*     // only crash-workaround
     doSelectionChanged();
     // only crash-workaround */
   }

   /**
    * Notification that inspectable source is being changed. 
   */
   public void inspectableChanged (XelfiDesignInspectable inspectable) {
     // saves new source and calls changeSelection
     this.inspectable = inspectable;
     selectionChanged();
   }

   /**
    * Notification from inspectable that says, hey, 
    * refresh your property values
   */ 
   public void propertiesChanged () {
     doPropertiesChanged();
   }

   /**
    * Returns currently attached inspectable source (or null)
   */
   public XelfiDesignInspectable getInspectable () {
     return inspectable;
   }

   /**
     * Returns a vector of components currently inspected in CI.
     * Returns null when no component is inspected now.
   */
   public Vector getInspectedComponents () {
     return commonProperties==null ? null 
                                   : commonProperties.getSelectedComponents();
   }

   /**
    * Responds to mouse,keyboard or other events in 
    * Component Inspector window.
   */
  public boolean handleEvent (Event e) {
    super.handleEvent(e);
     switch (e.id) {
       // TabControl event - page has changed, so change the content
       case TabControl.TAB_CHANGE:
         tabChanged(e);
         break;
     }
     return false;
   }

  /**
   * Delivers events for up/down keys to property panel.
   * Property panel is required to stop event propagation, so
   * cycle in event sending should not occur.
  */
  public boolean keyDown (Event e, int key) {
    super.keyDown(e, key);
    switch (key) {
      case Event.UP:
      case Event.DOWN:
        // up/down was pressed -
        // deliver it to the property panel
        propertyPanel.postEvent(e);
        break;
    }
    return false;
  }

   /**
     * Shows specified page in CI (one of properties, events or layouts).
     * Returns false when specified page doesn't exists.
   */
   public boolean setPage (int page) {
     // Debug.debug(Debug.INSPECTOR, "Page changed....");
     // check if we have content - if no than return
     if (commonProperties==null) return true;
     // check if change is not just paranoid creature
     if (getPage()==page) return true;
     // changes page and content
     if (commonProperties.setPage(page)==false) { 
       // set the page back in tab control
       setPageToTab(commonProperties.getPage());
       return false;
     }
     setPageToTab(page);

     // wait until threads can be STOPPED and
     // thrown to the binary space :-))
     selectPanel.waitUntilStoppable();
     if (pageThread!=null) pageThread.stop();
     // starts a new thread to perform time-consuming page-change
     pageThread = new PageThread(this, selectionThread);
     pageThread.start();

/*     // only crash-workaround
     doPageChanged();
     // only crash-workaround */

     return true;
   }

   /**
    * Returns currently selected page or -1 when no component
    * (and no page then) is inspected
   */
   public int getPage () {
     if (commonProperties==null) return -1;
     return commonProperties.getPage();  
   }

   /**
    * Response method to tab event which says "hey you, page has changed".
   */
   private void tabChanged (Event e) {
     // get index of new page
     int pageIndex = pageTab.getTabIndex();
     // decide what that index means
     int ourPage = -1;
     int[] pages = new int[3];
     pages[0] = XFDesignConstants.CI_PG_PROPERTIES;
     pages[1] = XFDesignConstants.CI_PG_EVENTS;
     pages[2] = XFDesignConstants.CI_PG_LAYOUT;
     for (int i=0; i<pages.length; i++)
       if (pageIndex==i) {
         ourPage = pages[i];
         break;
       }
     // set new page
     setPage(ourPage);
     // Debug.debug(Debug.INSPECTOR, "leaving setpage...");
   }

   private void setPageToTab (int page) {
     int tabIndex = -1;
     switch (page) {
       case XFDesignConstants.CI_PG_PROPERTIES:
         tabIndex = 0;
         break;
       case XFDesignConstants.CI_PG_EVENTS:
         tabIndex = 1;
         break;
       case XFDesignConstants.CI_PG_LAYOUT:
         tabIndex = 2;
         break;
     }     
     // set the page index into tab control
     pageTab.setTabIndex(tabIndex);
     // and repaint
     pageTab.repaint();
   }

   /**
    * Performs an action of changing the selection.
    * This is time-consuming , so should be runned in different thread
    * to avoid blocking user's input.
   */
   synchronized void doSelectionChanged () {
//     Debug.debug(Debug.INSPECTOR, "Entering selection....");
     if (commonProperties!=null) {
    // testing - commented, isSelectionDifferent needs repairing
/*       if (!isSelectionDifferent()) {
         // selection is the same as before - refresh values only
         propertiesChanged();
         return;
       } */
       // clear old content
       selectPanel.empty();
       propertyPanel.empty();
     }
     int   page = getPage();
     // property page by default
     if (page==-1) page = XFDesignConstants.CI_PG_PROPERTIES;
     // asks for set of selected components,
     // creates and finds common set of properties
     commonProperties =
         new XFCommonProperties(inspectable.getAllComponents(),
                                 inspectable.getSelectedComponents(), 
                                 page, pageTab);
     // make sub-windows change their content
     selectPanel.changeContent();
     propertyPanel.changeContent(commonProperties);
     // if common properties changed specified page to another
     // set tab control properly
     int newPage = commonProperties.getPage();
     if (page!=newPage)
       setPageToTab(newPage);
//     Debug.debug(Debug.INSPECTOR, "Exiting selection....");
   }

   /**
    * Actually performs the work of changing the content to diff. page
   */
   void doPageChanged () {
     propertyPanel.changeContent(commonProperties);
     selectPanel.changeContent();
   }

   /**
    * Refreshes property values
   */
   void doPropertiesChanged () {
     if ((selectionThread!=null) && (selectionThread.isAlive())) {
       // whooooo aaare these BAD BOYS?!!?
       // I'm working hard on changing the selection now,
       // so don't bother be with some chaotic property change!
       // I will do nothing - only return and you'll be out ! :-)))
       return;
     }
     propertyPanel.refreshValues();
   }

   private boolean isSelectionDifferent () {
     Vector selected = inspectable.getSelectedComponents();
     if ((oldSelection==null) ||
           (oldSelection.size()!=inspectable.getSelectedComponents().size())) {
       oldSelection = (Vector)selected.clone();
       return true;
     }
     for (int i=0; i<oldSelection.size(); i++)
       if (!oldSelection.elementAt(i).equals(selected.elementAt(i))) {
         oldSelection = (Vector)selected.clone();
         return true;
       }
     return false;
   }

}

