
package xelfi.design.inspector;

import java.awt.*;
import java.util.Vector;

import xelfi.awt.*;
import xelfi.design.components.*;
import xelfi.design.components.xawt.*;
import xelfi.util.Debug;

class XFMenuTreeElement implements XelfiTreeViewElement {

	// reference to associated menu component
	private MenuComponent				component;
	// pull state
	private boolean							pullState;
	// horizontal border
	private static final int	XBORDER = 10;
	// holds preferred size
	private Dimension					preferredSize;
	// current font
	private Font							font;
	// current font metrics - speeds up preferredSize method response
	private FontMetrics				fm;

	XFMenuTreeElement (MenuComponent component) {
		this(component, true, null);
	}

	XFMenuTreeElement (MenuComponent component, boolean pullState) {
		this(component, pullState, null);
	}

	XFMenuTreeElement (MenuComponent component, boolean pullState,
											Font font) {
		this.component = component;
		pull(pullState);
		setFont(font); 
	}

	/**
	 * Paints ifself at specified point (x-left y-center of element)
	Last change:  DF   16 Mar 97    5:27 pm
	*/
	public void paint (Graphics g, int x, int y) {
		g.drawString(getText(), x+2, 
									y+((fm.getAscent()+fm.getDescent())/2-fm.getDescent()));
	}

	/**
	 * Paints ifself with selection
	*/
	public void paintSelected (Graphics g, int x, int y) {
		g.drawRoundRect(x, y-fm.getHeight()/2, 
										fm.stringWidth(getText())+4, fm.getHeight(), 2, 2);
		paint(g, x, y);
	}

	/**
	 * Returns preferred size
	*/
	public Dimension preferredSize () {
		recalculateSize(getText());
		return preferredSize;
	}

	/**
	 * Returns true if element can have children, false otherwise
	*/
	public boolean canHaveChildren () {
		return getMenuComponent() instanceof MenuContainer;
	}

	/**
	 * Returns true if element is pulled down, false otherwise
	*/
	public boolean isPulled () {
		return pullState;
	}

	/**
	 * Sets element to specified state, true = pulled down
	*/
	public void pull (boolean state) {
		if (!canHaveChildren()) return;
		pullState = state;
	}

	void setFont (Font font) {
		this.font = font==null ? new Font("Helvetica", Font.PLAIN, 14) : font;
		fm = Toolkit.getDefaultToolkit().getFontMetrics(this.font);
		recalculateSize(getText());
	}

	String getText () {
		MenuComponent mc = getMenuComponent();
		if (mc instanceof MenuItem)
			return ((MenuItem)mc).getLabel();
		else
			if (mc instanceof MenuBar)
				return "Root";
		return null;
	}

	public void setText (String text) {
		MenuComponent mc = getMenuComponent();
//    if (mc instanceof XelfiDesignComponent) {
      // set "caption" property
      try {
        ((XelfiDesignComponent)mc).setProperty(0, text);
        recalculateSize(text);
      } catch (XFPropertyBadValueTypeException e) {};
//		}
	}

	/**
	 * Recalculates preferred size. We assume that fm is not null.
	*/
	private void recalculateSize (String text) {
		preferredSize = new Dimension(fm.stringWidth(text)+2*XBORDER,
																		fm.getHeight());
	}

	/**
	 * Returns reference to associated menu component.
	*/
	MenuComponent getMenuComponent () {
		return component;
	}

	/**
	 * Removes associated menu component from its parent
	 * (if it has parent)
	*/
	void remove () {
		MenuContainer parent = component.getParent();
		if (parent==null) return;
		parent.remove(getMenuComponent());
	}

	boolean insertAfter (XFMenuTreeElement mte) {
		boolean result = false;
		MenuContainer parent = getMenuComponent().getParent();
		if (parent==null) return result;
		// parent can be MenuBar or Menu Folder....
		if (parent instanceof MenuBar)
			result = insertAfterToMenuBar((MenuBar)parent, mte);
		else
			result = insertAfterToMenuFolder((Menu)parent, mte);
		return result;
	}	

	boolean insertInto (XFMenuTreeElement mte) {
		boolean result = false;
		// this instance must be either MenuBar or Menu Folder
		MenuComponent mc = getMenuComponent();
		if (mc instanceof MenuBar)
			result = insertIntoMenuBar(mte);
		else
			if (mc instanceof Menu)
				result = insertIntoMenuFolder(mte);
		return result;
	}	

	/**
	 * Inserts a menu component into this	instance 
	 * as its first child
	*/
	boolean insertAsFirst (XFMenuTreeElement mte) {
		// this menu component can be MenuBar or Menu Folder....
		MenuComponent mc = getMenuComponent();
		if (mc instanceof MenuBar)
			insertToMenuBarAt((MenuBar)mc, mte, 0);
		else
			insertToMenuFolderAt((Menu)mc, mte, 0);
		return true;
	}

	/**
	 * Inserts specified menu component into MenuBar parent
	 * after menu component associated with this instance
	*/
	private boolean insertAfterToMenuBar (MenuBar parent, 
																				XFMenuTreeElement mte) {
		// ensures that new menu comp is instance of menu
		if (!(mte.getMenuComponent() instanceof Menu)) return false;
		// inserts after
		int foundIndex = findIndexInMenuBar(parent);
		if (foundIndex<0) return false;
		insertToMenuBarAt(parent, mte, foundIndex+1);
		return true;
	}

	/**
	 * Inserts specified menu component into MenuFolder parent
	 * after menu component associated with this instance
	*/
	private boolean insertAfterToMenuFolder (Menu parent, 
																						XFMenuTreeElement mte) {
		// ensures that new menu comp is instance of menu item
		if (!(mte.getMenuComponent() instanceof MenuItem)) return false;
		// inserts after
		int foundIndex = findIndexInMenuFolder(parent);
		if (foundIndex<0) return false;
		insertToMenuFolderAt(parent, mte, foundIndex+1);
		return true;
	}

	/**
	 * Inserts item as a last item into menu component 
	 * associated with this instance
	*/
	private boolean insertIntoMenuFolder (XFMenuTreeElement mte) {
		// ensures that new menu comp is instance of MenuItem
		MenuComponent mc = mte.getMenuComponent();
		if (!(mc instanceof MenuItem)) return false;
		((Menu)getMenuComponent()).add((MenuItem)mc);
		return true;
	}


	/**
	 * Inserts item as a last item into menu component 
	 * associated with this instance
	*/
	private boolean insertIntoMenuBar (XFMenuTreeElement mte) {
		// ensures that new menu comp is instance of Menu
		MenuComponent mc = mte.getMenuComponent();
		if (!(mc instanceof Menu)) return false;
		((MenuBar)getMenuComponent()).add((Menu)mc);
		return true;
	}

	/**
	 * Finds index of menu component associated with this instance in
	 * MenuBar parent
	*/
	private int findIndexInMenuBar(MenuBar parent) {
		int foundIndex = -1;
		for (int i=0; i<parent.countMenus(); i++) {
			if (parent.getMenu(i)==(Menu)getMenuComponent()) {
				foundIndex = i;
				break;			
			}
		}
		return foundIndex;
	}


	/**
	 * Finds index of menu component associated with this instance in
	 * MenuFolder parent
	*/
	private int findIndexInMenuFolder(Menu parent) {
		int foundIndex = -1;
		for (int i=0; i<parent.countItems(); i++) {
			if (parent.getItem(i)==(MenuItem)getMenuComponent()) {
				foundIndex = i;
				break;			
			}
		}
		return foundIndex;
	}

	/**
	 * Helper method. Inserts menu to menu bar at specified index.
	 * (Yep, this is a hack, it should be in AWT!)
	*/
	private void insertToMenuBarAt (MenuBar parent, XFMenuTreeElement mte,
																	int index) {
		// temporary delete menus after index
		Vector savedMenus = new Vector();
		int parentMenuCount = parent.countMenus();
    for (int i=parentMenuCount-1; i>=index; i--) {
			savedMenus.addElement((Object)(parent.getMenu(i)));
			parent.remove(i);
		}
		// add specified menu
		parent.add((Menu)mte.getMenuComponent());
		// insert previously removed menus again
    for (int i=savedMenus.size()-1; i>=0; i--)
			parent.add((Menu)(savedMenus.elementAt(i)));
	}

	/**
	 * Helper method. Inserts menuitem to menu folder at specified index.
	 * (Yep, this is a hack, it should be in AWT!)
	*/
	private void insertToMenuFolderAt (Menu parent, XFMenuTreeElement mte,
																			int index) {
		// temporary delete menus after index
		Vector savedMenus = new Vector();
		int parentMenuCount = parent.countItems();
		for (int i=parentMenuCount-1; i>=index; i--) {
			savedMenus.addElement((Object)(parent.getItem(i)));
			parent.remove(i);
		}
		// add specified menu
		parent.add((MenuItem)mte.getMenuComponent());
		// insert previously removed menus again
		for (int i=savedMenus.size()-1; i>=0; i--)
			parent.add((MenuItem)(savedMenus.elementAt(i)));
	}
	
}


