package xelfi.editor.keys;

import java.util.Hashtable;
import java.util.Vector;
import java.util.Enumeration;
import java.io.StringBufferInputStream;
import java.io.InputStream;

class KeyConfig implements KeyConfigConstants {
  static Hashtable alias;

  /** table */
  static Hashtable actions;
  /** key parser */
  static Key keyParser;
  /** key config parser */
  static KeyConfig keyConfigParser;
  /** imported packages */
  static Vector imports;

  /** Is called when a request for object import of package is made.
  * @param s package name
  */
  static void importPackage (String s) {
    imports.addElement (s);
  }

  /** Is called when new action is created and its result assigned to
  * identifier s.
  * @param s the identifier
  * @param c the command to call
  * @exception ParseError if c does not contain valid action
  */
  static void storeAction (String s, Vector c) throws ParseError {
    actions.put (s, toHandlerList (c));
  }

  /** Should store key and action that should be called.
  * @param key string that describes the key
  * @param list list of commands that should be called
  * @exception ParseError if the keys is not correct
  */
  static void key (String key, Vector list) throws ParseError {
    StringBufferInputStream is = new StringBufferInputStream (key);

    if (keyParser == null) keyParser = new Key (is);
    else keyParser.ReInit (is);

    KeyCode kc = keyParser.key (key);

    KeyActionMacro v = toHandlerList (list);

    if (!KeyOperator.addKey (kc, v)) {
      // cannot add this key
      throw new ParseError ("Cannot add key " + kc);
    }
  }
  /** Converts list of ConfigCommands to list of KeyActionHandlers
  * @param list list
  * @return new list
  * @exception ParseError if an error occures
  */
  static KeyActionMacro toHandlerList (Vector list) throws ParseError {
    int s = list.size ();
    KeyActionMacro v = new KeyActionMacro (s);

    for (int i = 0; i < s; i++) {
      ConfigCommand c = (ConfigCommand)list.elementAt (i);
      v.addElement (toHandler (c));
    }
    return v;
  }

  /** Converts command to instance of KeyActionHandler.
  * @param c the command
  * @return new instance of KeyActionHandler initialized with
  *   appropriate parameters
  * @exception ParseError if it is not possible to create this instance
  */
  static KeyActionHandler toHandler (ConfigCommand c) throws ParseError {
    // if the command is call of already predefinded action
    KeyActionHandler h = (KeyActionHandler)actions.get (c.name);
    if (h != null) {
      if (c.param.size () > 0) {
        // passing parameters to already defined action
        throw new ParseError ("Macro cannot accept parameters [" + c.name + "]");
      }
      return h;
    }
    // otherwise we must find the class by its name
    try {
      Class cl = findClass (c.name);
      h = (KeyActionHandler)cl.newInstance ();
    } catch (Exception ex) {
      throw new ParseError ("Cannot create handler for " + c);
    }
    try {
      h.init (c.param);
    } catch (IllegalArgumentException ex) {
      throw new ParseError ("Parameters are not valid for " + c);
    }
    return h;
  }

  /** Scans paths for class name.
  * @param name the name of the class
  * @return the class
  * @exception ClassNotFoundException if the class has not been found
  */
  static Class findClass (String name) throws ClassNotFoundException {
    /* basic try, the name contains whole path */
    try {
      return Class.forName (name);
    } catch (Exception ex) {
    }
    // prefix the editor
    try {
      return Class.forName ("xelfi.editor.actions." + name);
    } catch (Exception ex) {
    }
    // look through imports
    Enumeration en = imports.elements ();
    while (en.hasMoreElements ()) {
      try {
        return Class.forName (en.nextElement ().toString () + '.' + name);
      } catch (Exception ex) {
      }
    }
    throw new ClassNotFoundException (name);
  }

  /** Registers symbol as used.
  * @param s the symbol
  */
  static void registerSymbol (String s) throws ParseError {

    addSymbol (s, s);
  }

  /** Assigns value to the symbol
  * @param s the symbol
  * @param val the value
  */
  static void addSymbol (String s, String val) throws ParseError {
    String x = (String)alias.get (s);
    if (x != null) throw new ParseError ("Symbol " + s + " has already been defined");
    alias.put (s, val);
  }

  /** Handles the parsing.
  * @param is input stream
  * @exception ParseError if an error occures
  */
  public static void parseConfig (InputStream is) throws ParseError {
    if (keyConfigParser != null) {
      keyConfigParser.ReInit (is);
    } else {
      keyConfigParser = new KeyConfig (is);
    }
    alias = new Hashtable ();
    actions = new Hashtable ();
    imports = new Vector ();
    try {
      keyConfigParser.config ();
    } catch (ParseError ex) {
      if (ex.getMessage () == null) {
        // new get message
        throw new ParseError ("Syntax error");
      } else {
        throw ex;
      }
    }
  }

  public static void main(String args[]) {
    java.io.InputStream is;
    if (args.length == 0) {
      System.out.println("Key:  Reading from standard input . . .");
      is = System.in;
    } else if (args.length == 1) {
      System.out.println("Key:  Reading from file " + args[0] + " . . .");
      try {
        is = new java.io.FileInputStream(args[0]);
      } catch (java.io.FileNotFoundException e) {
        System.out.println("Key:  File " + args[0] + " not found.");
        return;
      }
    } else {
      System.out.println("Key:  Usage is one of:");
      System.out.println("         java Key < inputfile");
      System.out.println("OR");
      System.out.println("         java Key inputfile");
      return;
    }
    try {
      KeyOperator.load (is);

      System.out.println("Key: ");
      System.out.println (KeyOperator.tableToString ());
    } catch (EditorKeyException e) {
      System.out.println("Key:  " + e.getMessage ());
      e.printStackTrace ();
    }
  }

/* Grammer begins here */
  static final public void config() throws ParseError {
    importSection();
    actionSection();
    keySection();
    jj_consume_token(0);
  }

  static final public void importSection() throws ParseError {
  String s;
  boolean pck;
    label_1:
    while (true) {
      if (jj_mask_0[getToken(1).kind]) {
        ;
      } else {
        break label_1;
      }
      jj_consume_token(IMPORT);
      s = name();
                          pck = false;
      if (jj_mask_1[getToken(1).kind]) {
        jj_consume_token(DOT);
        jj_consume_token(AST);
                                                         pck = true;
      } else {
        ;
      }
      jj_consume_token(STRED);
    if (pck) {
      // add package to imports
      importPackage (s);
    } else {
      // import of one class
      int last = s.lastIndexOf ('.');
      String rest = s.substring (last + 1);
      String old = (String)alias.get (rest);
      if (old != null) {
        throw new ParseError ("Import of " + s + " conflicts with " + old);
      }
      alias.put (rest, s);
    }
    }
  }

  static boolean[] jj_mask_0 = new boolean[25];
  static {
    jj_mask_0[IMPORT] = true;
  }
  static boolean[] jj_mask_1 = new boolean[25];
  static {
    jj_mask_1[DOT] = true;
  }
  
  static final public void actionSection() throws ParseError {
  Token t;
  Vector c;
    label_2:
    while (true) {
      if (jj_mask_2[getToken(1).kind]) {
        ;
      } else {
        break label_2;
      }
      jj_consume_token(MACRO);
      t = jj_consume_token(IDENTIFIER);
      c = commandList();
       registerSymbol (t.image); storeAction (t.image, c);
    }
  }

  static boolean[] jj_mask_2 = new boolean[25];
  static {
    jj_mask_2[MACRO] = true;
  }
  
  static final public void keySection() throws ParseError {
  Token t;
  Vector v;
    label_3:
    while (true) {
      if (jj_mask_3[getToken(1).kind]) {
        ;
      } else {
        break label_3;
      }
      jj_consume_token(KEY);
      t = jj_consume_token(KEY_LITERAL);
      v = commandList();
                                                  key (t.image, v);
    }
  }

  static boolean[] jj_mask_3 = new boolean[25];
  static {
    jj_mask_3[KEY] = true;
  }
  
  static final public String name() throws ParseError {
  Token t;
  StringBuffer sb;
    if (jj_2_2(2)) {
      t = jj_consume_token(IDENTIFIER);
                                  sb = new StringBuffer (t.image);
      label_4:
      while (true) {
        jj_consume_token(DOT);
        t = jj_consume_token(IDENTIFIER);
                                            sb.append ('.'); sb.append (t.image);
        if (jj_2_1(2)) {
          ;
        } else {
          break label_4;
        }
      }
    return sb.toString ();
    } else {
      if (jj_mask_4[getToken(1).kind]) {
        t = jj_consume_token(IDENTIFIER);
    String s = (String) alias.get (t.image);
    return s == null ? t.image : s;
      } else {
        jj_consume_token(-1);
        throw new ParseError();
      }
    }
  }

  static boolean[] jj_mask_4 = new boolean[25];
  static {
    jj_mask_4[IDENTIFIER] = true;
  }
  
  static final public Vector params() throws ParseError {
  Token t;
  Vector v = new Vector ();
  Object o;
    label_5:
    while (true) {
      if (jj_mask_5[getToken(1).kind]) {
        ;
      } else {
        break label_5;
      }
      if (jj_mask_8[getToken(1).kind]) {
        t = jj_consume_token(STRING_LITERAL);
                          o = t.image.substring (1, t.image.length () - 1);
      } else {
        if (jj_mask_7[getToken(1).kind]) {
          t = jj_consume_token(CHARACTER_LITERAL);
                               o = new Character (t.image.charAt (1));
        } else {
          if (jj_mask_6[getToken(1).kind]) {
            t = jj_consume_token(INTEGER_LITERAL);
     try {
       o = Integer.valueOf (t.image);
     } catch (NumberFormatException ex) {
       // this cannot happen, but
       throw new ParseError ("Wrong number format: " + t.image);
     }
          } else {
            jj_consume_token(-1);
            throw new ParseError();
          }
        }
      }
       v.addElement (o);
    }
                                return v;
  }

  static boolean[] jj_mask_5 = new boolean[25];
  static {
    jj_mask_5[INTEGER_LITERAL] =
    jj_mask_5[CHARACTER_LITERAL] =
    jj_mask_5[STRING_LITERAL] = true;
  }
  static boolean[] jj_mask_6 = new boolean[25];
  static {
    jj_mask_6[INTEGER_LITERAL] = true;
  }
  static boolean[] jj_mask_7 = new boolean[25];
  static {
    jj_mask_7[CHARACTER_LITERAL] = true;
  }
  static boolean[] jj_mask_8 = new boolean[25];
  static {
    jj_mask_8[STRING_LITERAL] = true;
  }
  
  static final public ConfigCommand command() throws ParseError {
  String s;
  Vector p;
    s = name();
    p = params();
    jj_consume_token(STRED);
                                  return new ConfigCommand (s, p);
  }

  static final public Vector commandList() throws ParseError {
  Vector v = new Vector ();
  ConfigCommand c;
    jj_consume_token(BEGIN);
    label_6:
    while (true) {
      if (jj_mask_9[getToken(1).kind]) {
        ;
      } else {
        break label_6;
      }
      c = command();
                        v.addElement (c);
    }
    jj_consume_token(END);
    if (jj_mask_10[getToken(1).kind]) {
      jj_consume_token(STRED);
    } else {
      ;
    }
                                                              return v;
  }

  static boolean[] jj_mask_9 = new boolean[25];
  static {
    jj_mask_9[IDENTIFIER] = true;
  }
  static boolean[] jj_mask_10 = new boolean[25];
  static {
    jj_mask_10[STRED] = true;
  }
  
  static final private boolean jj_2_1(int xla) throws ParseError {
    jj_la = xla; jj_lastpos = jj_scanpos = token;
    return !jj_3_1();
  }

  static final private boolean jj_2_2(int xla) throws ParseError {
    jj_la = xla; jj_lastpos = jj_scanpos = token;
    return !jj_3_2();
  }

  static final private boolean jj_3_1() throws ParseError {
    if (jj_scan_token(DOT)) return true;
    if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
    if (jj_scan_token(IDENTIFIER)) return true;
    if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
    return false;
  }

  static final private boolean jj_3_2() throws ParseError {
    if (jj_scan_token(IDENTIFIER)) return true;
    if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
    Token xsp;
    if (jj_3_1()) return true;
    if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
    while (true) {
      xsp = jj_scanpos;
      if (jj_3_1()) { jj_scanpos = xsp; break; }
      if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
    }
    return false;
  }

  static private boolean jj_initialized_once = false;
  static public KeyConfigTokenManager token_source;
  static public Token token;
  static private Token jj_scanpos, jj_lastpos;
  static private int jj_la;
  static public boolean lookingAhead = false;
  static private boolean jj_semLA;

  public KeyConfig(java.io.InputStream stream) {
    if (jj_initialized_once) {
      System.out.println("ERROR: Second call to constructor of static parser.  You must");
      System.out.println("       either use ReInit() or set the JavaCC option STATIC to false");
      System.out.println("       during parser generation.");
      throw new Error();
    }
    jj_initialized_once = true;
    ASCII_UCodeESC_CharStream input_stream = new ASCII_UCodeESC_CharStream(stream, 1, 1);
    token_source = new KeyConfigTokenManager(input_stream);
    token = new Token();
  }

  static public void ReInit(java.io.InputStream stream) {
    ASCII_UCodeESC_CharStream input_stream = new ASCII_UCodeESC_CharStream(stream, 1, 1);
    token_source.ReInit(input_stream);
    token = new Token();
  }

  public KeyConfig(KeyConfigTokenManager tm) {
    if (jj_initialized_once) {
      System.out.println("ERROR: Second call to constructor of static parser.  You must");
      System.out.println("       either use ReInit() or set the JavaCC option STATIC to false");
      System.out.println("       during parser generation.");
      throw new Error();
    }
    jj_initialized_once = true;
    token_source = tm;
    token = new Token();
  }

  public void ReInit(KeyConfigTokenManager tm) {
    token_source = tm;
    token = new Token();
  }

  static final private Token jj_consume_token(int kind) throws ParseError {
    if (token.next != null) token = token.next;
    else token = token.next = token_source.getNextToken();
    if (token.kind == kind) {
      return token;
    }
    jj_token_error_setup(token, kind);
    throw new ParseError();
  }

  static final private boolean jj_scan_token(int kind) throws ParseError {
    if (jj_scanpos == jj_lastpos) {
      jj_la--;
      if (jj_scanpos.next == null) {
        jj_lastpos = jj_scanpos = jj_scanpos.next = token_source.getNextToken();
      } else {
        jj_lastpos = jj_scanpos = jj_scanpos.next;
      }
    } else {
      jj_scanpos = jj_scanpos.next;
    }
    return (jj_scanpos.kind != kind);
  }

  static final public Token getNextToken() throws ParseError {
    if (token.next != null) token = token.next;
    else token = token.next = token_source.getNextToken();
    return token;
  }

  static final public Token getToken(int index) throws ParseError {
    Token t = lookingAhead ? jj_scanpos : token;
    for (int i = 0; i < index; i++) {
      if (t.next != null) t = t.next;
      else t = t.next = token_source.getNextToken();
    }
    return t;
  }

  static final String jj_add_escapes(String str) {
    String retval = "";
    char ch;
    for (int i = 0; i < str.length(); i++) {
      ch = str.charAt(i);
      if (ch == '\b') {
        retval += "\\b";
      } else if (ch == '\t') {
        retval += "\\t";
      } else if (ch == '\n') {
        retval += "\\n";
      } else if (ch == '\f') {
        retval += "\\f";
      } else if (ch == '\r') {
        retval += "\\r";
      } else if (ch == '\"') {
        retval += "\\\"";
      } else if (ch == '\'') {
        retval += "\\\'";
      } else if (ch == '\\') {
        retval += "\\\\";
      } else if (ch < 0x20 || ch > 0x7e) {
        String s = "0000" + Integer.toString(ch, 16);
        retval += "\\u" + s.substring(s.length() - 4, s.length());
      } else {
        retval += ch;
      }
    }
    return retval;
  }

  static final private void jj_token_error_setup(Token current, int kind) throws ParseError {
    int line = current.beginLine, column = current.beginColumn;
    System.out.println("Parse error at line " + line + ", column " + column + ".  Encountered: " + jj_add_escapes((current.kind == 0) ? tokenImage[0] : current.image));
  }

  static final public void enable_tracing() {
  }

  static final public void disable_tracing() {
  }

}
