/*
* Xelfi programming environment
*/

package xelfi.xedit;
import xelfi.editor.*;
import java.util.NoSuchElementException;
/**
* Class that handles comments. Remembers the position of
* all comments in document.
*
* @version 1.0
* @author Jaroslav Tulach
*/

public class Comments extends Object implements ActionPostListener {
  /** container of all special positions in the document */
  PositionContainer container = new PositionContainer ();

  /** document */
  private Document doc;

  /** Creates data structure for document d.
  * @param d the document
  */
  public Comments (Document d) {
    addBlock (d, false);
    d.addActionPostListener (this);
    doc = d;
  }

  /** Closes.
  */
  public void close () {
    doc.removeActionPostListener (this);
  }

  /** Finds info for specified position.
  * @param p the position to find info for
  * @return the info for this position
  */
  synchronized CommentMark findMark (Position p) {
    try {
      // the position with previous mark for p
      Position prev = container.previousOrEqual (p);

      // info for position prev (should exists)
      CommentMark cm = (CommentMark) container.getInfo (prev);

      // strings and line comments cannot propagete after
      // its line
      switch (cm.type) {
      case CommentMark.STRING:
      case CommentMark.LINE:
        if (!p.sameLine (cm.pos)) {
          return new CommentMark (cm.pos, cm.type, cm.area);
        }
      }
      return cm;
    } catch (NoSuchElementException ex) {
      // the position p is at the begin of document
      return new CommentMark (
        p.getDocument ().getBegin (), CommentMark.NOTHING, CommentInfo.NOTHING
      );
    }
  }

  /** Adds new position that finds in the block b.
  * @param b the block to scan for new positions
  * @param red true if modified block should be redrawn
  * @return the block that has been (or should be) redrawn or null
  */
  synchronized Block addBlock (Block b, boolean red) {
//    xelfi.util.Debug.debug ("Block: \'" + b + '\'');
    // the acceptor for finding marks
    CommentsAcceptor ca = new CommentsAcceptor (b);

    // search for changed mark
    CommentMark mark = ca.findNext ();

    if (mark == null) {
      // no mark has been found in inserted block
      return null;
    }

    // this is the color used at position of newly inserted mark
    // now we will try to compare it
    CommentMark oldMark = findMark (mark.pos);

    Position beginRedraw = null;
    for (;;) {
      // inserts this mark into positions
      container.insert (mark.pos, mark);
      // now tests if the mark's area remains same
      mark.area = oldMark.area;
      if (setType (mark, oldMark)) {
        // area's type has been changed
        if (beginRedraw == null) {
          beginRedraw = mark.pos;
        }
      }
      // go to the next mark
      CommentMark next = ca.findNext ();
      oldMark = mark;
    if (next == null) break;
      // next positions
      mark = next;
    }

    // test if something has not changed after the inserted block
    // like adding * before //
    /* [PENDING]
    */

    // should we redraw part of text?
    if (beginRedraw == null) return null;

    return scanChanges (beginRedraw, mark, red);
  }

  /** Scans for changes in block
  * @param mark the position to test from
  * @return the block that has been or should been redrawn
  */
  Block scanChanges (CommentMark mark) {
    return scanChanges (mark.pos, mark, true);
  }

  /** Scans for changes in block
  * @param beginRedraw the first position to redraw
  * @param mark the position to test from
  * @param red true if we should redraw the block
  * @return the block that has been or should been redrawn
  */
  Block scanChanges (Position beginRedraw, CommentMark mark, boolean red) {
    // oldMark = mark
    CommentMark oldMark = mark;

    Position end;
    try {
      mark = (CommentMark) container.getInfo (container.next (oldMark.pos));
      while (setType (mark, oldMark)) {
        // the mark's area field has been changed
        oldMark = mark;
        mark = (CommentMark) container.getInfo (container.next (oldMark.pos));
      }
      // everything from mark.pos remains the same
      end = mark.pos;
    } catch (NoSuchElementException ex) {
      // changes till the end of document
      end = mark.pos.getDocument ().getEnd ();
    }

    try {
      Block block = new Block (beginRedraw, end);
//      xelfi.util.Debug.debug ("Redraw: \'" + block + '\'');
      if (red) {
        // redraw the block
        block.redraw ();
      }
      return block;
    } catch (NoSuchElementException ex) {
      return null;
    }
  }

  /** Sets type of area for the mark.
  * @param mark the mark to set the area to
  * @param prev the previous one (already set)
  * @return true, if the area type of mark changes
  */
  boolean setType (CommentMark mark, CommentMark prev) {
    // the resulting area type
    byte area = CommentInfo.NOTHING;

    switch (prev.area) {
    case CommentInfo.BLOCK_COMMENT:
      // we are in comment
      if (mark.type == CommentMark.END) {
        // only end of comment can change it
        area = CommentInfo.NOTHING;
      } else {
        area = prev.area;
      }
      break;
    case CommentInfo.STRING:
      // we are in string
      if (mark.pos.sameLine (prev.pos) && mark.type == CommentMark.STRING) {
        // the " ends the string
        area = CommentInfo.NOTHING;
        break;
      }
      // otherwise everything behaves line in BLOCK_LINE case
    case CommentInfo.LINE_COMMENT:
      // we are in line block or string
      if (mark.pos.sameLine (prev.pos)) {
        // if we are on the same line as begining than we
        // have no change do anything => we are ignored
        area = prev.area;
        break;
      }
      // otherwise everything behaves line in NOTHING case:
    case CommentInfo.NOTHING:
      // we are not in any area
      // everything changes it somehow:
      switch (mark.type) {
      case CommentMark.BEGIN:
        area = CommentInfo.BLOCK_COMMENT;
        break;
      case CommentMark.END:
        area = CommentInfo.NOTHING;
        break;
      case CommentMark.LINE:
        area = CommentInfo.LINE_COMMENT;
        break;
      case CommentMark.STRING:
        area = CommentInfo.STRING;
        break;
      }
      break;
    }
    boolean res = mark.area != area;
    mark.area = area;
//    xelfi.util.Debug.debug ("Set type at " + mark.pos + " to " + mark.area);
    return res;
  }

  /** From mark creates info.
  * @param mark the mark
  * @return the info
  */
  public CommentInfo markToInfo (CommentMark mark) {
    try {
      Position end = container.next (mark.pos);
      CommentMark cm = (CommentMark) container.getInfo (end);
      switch (mark.area) {
      case CommentInfo.BLOCK_COMMENT:
        //end = end.next ();
        break;
      case CommentInfo.STRING:
        if (cm.type == CommentMark.STRING) {
          // the string can be finish here
          if (cm.pos.sameLine (mark.pos)) {
            end = cm.pos.next ();
            break;
          }
        }
      case CommentInfo.LINE_COMMENT:
        // the string or line comment goes to the end of line
        end = mark.pos.endOfLine ();
        break;
      // default = NOTHING, the end is at the next mark
      }
      return new CommentInfo (new Block (mark.pos, end), mark.area);
    } catch (NoSuchElementException ex) {
      switch (mark.area) {
      case CommentInfo.LINE_COMMENT:
      case CommentInfo.STRING:
        // the same info remains till the end of line
        return new CommentInfo (
          new Block (mark.pos, mark.pos.endOfLine ()),
          mark.area
        );
      default:
        // the same info remains till the end of document
        return new CommentInfo (
          new Block (mark.pos, mark.pos.getDocument ().getEnd ()),
          mark.area
        );
      }
    }
  }

  /** Finds previous mark to this position
  * @param p the position
  * @return the mark that applies to the position before p
  */
  public CommentMark previousMark (Position p) {
    try {
      Position pos = p.previousChar ();
//      xelfi.util.Debug.debug ("Prev position " + pos);
      CommentMark com = findMark (pos);
//      xelfi.util.Debug.debug ("Prev mark at " + com.pos + " is " + com.area);
      switch (com.area) {
      case CommentInfo.LINE_COMMENT:
      case CommentInfo.STRING:
        if (!com.pos.sameLine (p)) {
          // the info does not goes till the position
          throw new EndOfTextException ();
        }
      }
      return com;
    } catch (EndOfTextException ex) {
      // the position p is at the begin of document
      return new CommentMark (
        p.getDocument ().getBegin (), CommentMark.NOTHING, CommentInfo.NOTHING
      );
    }
  }
  /** Test if this position is in the comment or not.
  * @param p the position
  * @return the CommentInfo for the position
  * @see CommentInfo
  */
  synchronized public CommentInfo find (Position p) {
    try {
      // the previous mark and its mark info
      Position prev = container.previousOrEqual (p);
      CommentMark cm = (CommentMark) container.getInfo (prev);

//      xelfi.util.Debug.debug ("Find for " + p);
      if (p.equals (prev)) {
//        xelfi.util.Debug.debug ("Equal:" + cm.type);
        switch (cm.type) {
        case CommentMark.STRING:
          {
//            xelfi.util.Debug.debug ("Mark for " + p);
            CommentMark mark = previousMark (p);
//            xelfi.util.Debug.debug ("Mark is " + mark.area);
            if (mark.area == CommentInfo.STRING) {
              // this is the end of string
//              xelfi.util.Debug.debug ("String: " + p);
              return new CommentInfo (new Block (p, p.next ()), CommentInfo.STRING);
            }
            // otherwise handle normally
          }
          break;
        case CommentMark.END:
          {
            CommentMark mark = previousMark (p);
            if (mark.area == CommentInfo.BLOCK_COMMENT) {
              // end of comment
              return new CommentInfo (new Block (p, p.next ()), CommentInfo.BLOCK_COMMENT);
            }
          }
          break;
        }
      }

      CommentInfo ci = markToInfo (cm);

      if (ci.comment.containsOrBegins (p)) {
        return ci;
      } else {
        // end will contain the position of next mark
        Position end;
        try {
          end = container.next (prev);
        } catch (NoSuchElementException ex) {
          end = prev.getDocument ().getEnd ();
        }
        // the empty block
        return new CommentInfo (
          new Block (ci.comment.getEnd (), end), CommentInfo.NOTHING
        );
      }
    } catch (NoSuchElementException ex) {
      // nearly at the top of document, no marks before => empty text
      try {
        Position end;
        try {
          end = container.nextOrEqual (p);
        } catch (NoSuchElementException ex2) {
          end = p.getDocument ().getEnd ();
        }
        Block b = new Block (p.getDocument ().getBegin (), end);
        return new CommentInfo (b, CommentInfo.NOTHING);
      } catch (RuntimeException ex2) {
        ex2.printStackTrace ();
        throw ex2;
      }
    }
  }

  /** Notifies that an insert has been done at position p.
  * @param n the begin of inserted text
  * @param p the end of inserted text
  */
  public synchronized void notifyInsert (Position n, Position p) {
    // test if the insert has deleted any special marks on position n
    Position changed = null;
    try {
      // test if the insert was not into /* or //
      Position beg = n.previous ();
      if (container.isMember (beg)) {
        CommentMark cm = (CommentMark)container.getInfo (beg);
        switch (cm.type) {
        case CommentMark.BEGIN:
        case CommentMark.LINE:
          changed = beg;
          container.delete (beg);
          break;
        }
      }
    } catch (NoSuchElementException ex) {
    } catch (EndOfTextException ex) {
    }
    try {
      // insert into */ ?
      Position beg = p;
      if (container.isMember (beg)) {
        CommentMark cm = (CommentMark)container.getInfo (beg);
        if (cm.type == CommentMark.END) {
          container.delete (beg);
          if (changed == null) {
            changed = beg;
          }
        }
      }
    } catch (NoSuchElementException ex) {
    }
    if (addBlock (new Block (n, p), true) == null && changed != null) {
      scanChanges (changed, findMark (changed), true);
    }
  }

  /** Notifies a delete
  * @param p the position where a delete occures
  * @param s the string with characters that has been deleted
  */
  public void notifyDelete (Position p, String s) {
    // removes all marks about the positions that has been deleted
    // position before

    Position change = null;
    boolean del = (s.indexOf ('/') >= 0) || (s.indexOf ('*') >= 0) || (s.indexOf ('\"') >= 0);
    if (del) {
      try {
        // /*, //, " at delete position
        if (container.isMember (p)) {
          CommentMark cm = (CommentMark)container.getInfo (p);
          switch (cm.type) {
          case CommentMark.END:
            change = p;
            try {
              change = p.previous ();
            } catch (EndOfTextException ex) {
            }
            container.delete (p);
            break;
          case CommentMark.BEGIN:
          case CommentMark.LINE:
            change = p;
            container.delete (p);
            break;
          case CommentMark.STRING:
            change = p;
            container.delete (p);
            break;
          }
        }
      } catch (NoSuchElementException ex) {
      }

      try {
        // /*, //, */ at delete position
        Position beg = p.previous ();
        if (container.isMember (beg)) {
          CommentMark cm = (CommentMark)container.getInfo (beg);
          switch (cm.type) {
          case CommentMark.BEGIN:
          case CommentMark.LINE:
          case CommentMark.END:
            change = beg;
            container.delete (beg);
            break;
          }
        }
      } catch (NoSuchElementException ex) {
      } catch (EndOfTextException ex) {
      }
    }

    Block b = new Block (p, p.next ());
    // rescans the block
    if (addBlock (b, true) == null && change != null) {
      scanChanges (change, findMark (change), true);
    }
  }

  /** Handle event allows sending some predefined messages to position
  * and by throw the document to ActionController
  * @param p the position a message is sent to
  * @param e the event (the same as in java.awt)
  * @return true to signal successful handling (then no next distribution
  * to other controlers is done)
  * @see java.awt.Event
  public boolean handleEvent (Position p, Event e) {
    if (e.id == Document.SEARCH && e.target == getClass ()) {
      e.arg = this;
      return true;
    }
    return false;
  }
  */
}
























